
#include "anyl.h"
#include "disp.h"
#include "config.h"
// Timeout is only needed to check for other tasks...use is similar to YIELD except returns to current position
#define ANYL_TIMEOUT 1

xTaskHandle anyl_task_handle;
anyl_result_t anyl_result;

static volatile struct {
	unsigned int mode:2,
		changed:1, // 1 if mode has changed
		go:1;
	u8 avgs;
} cstate;

static xTaskHandle _monitor, _directmonitor, this;

static struct {
	unsigned int new:1;
	u8 codebuf[128*3];
	u8 *next;
	int collected;
} anstate;

static void setresult(float c)
{
	anyl_result.status=ANYL_STAT_DATA;
	anyl_result.newstatus=1;
	anyl_result.newresult=1;
	anyl_result.floatfmt=1;
	anyl_result.res.f=c;
	// This really does nothing unless task has been supsended
	if (_monitor)
		vTaskResume(_monitor);
}

static void setintresult(s32 c)
{
	anyl_result.status=ANYL_STAT_DATA;
	anyl_result.newstatus=1;
	anyl_result.newresult=1;
	anyl_result.floatfmt=0;
	anyl_result.res.i=c;
	// This really does nothing unless task has been supsended
	if (_monitor)
		vTaskResume(_monitor);
}

static s32 get_from_buf(u8 **ptr)
{
	s32 c;
	u8 *cptr=(u8 *)(&c);
	
	cptr[2]=*(*ptr)++;
	cptr[1]=*(*ptr)++;
	cptr[0]=*(*ptr)++;
	if (cptr[2]&0x80) cptr[3]=0xff;
	else cptr[3]=0;
	return c;
}

// standard deviation on packed code buffer
static double stddev(u8 *buf, int n)
{
	int i;
	double mean,dev,var;
	u8 *p=buf;

	mean=0;
	for (i=0;i<n;++i) 
		mean+=get_from_buf(&p);
	mean=mean/n;

	p=buf;
	var=0;
	for (i=0;i<n;++i) {
		dev=get_from_buf(&p)-mean;
		var+=dev*dev;
	}
	return sqrtf(var/n);
}

// average of packed buffer
static double avg(u8 *buf, int n)
{
	int i;
	double mean;
	u8 *p=buf;

	mean=0;
	for (i=0;i<n;++i) 
		mean+=get_from_buf(&p);
	return mean/n;
}

static int collect(int n)
{
	s32 code;

	ads_get(&code);		// from ads.c
	anyl_result.direct=code;
	anyl_result.newdirect=1;
	// This really does nothing unless task has been supsended
	//if (_directmonitor)
	//	vTaskResume(_directmonitor);
	if (cstate.changed) {
		anstate.new=1;
		cstate.changed=0;
		anyl_result.collected=0;
		anstate.next=anstate.codebuf;
		if (anyl_result.status==ANYL_STAT_NODATA||
			anyl_result.status==ANYL_STAT_DATA)
			anyl_result.status=ANYL_STAT_COLLECTING;
		anyl_result.newstatus=1;
	}
	*(anstate.next++)=(code>>16)&0xff;
	*(anstate.next++)=(code>>8)&0xff;
	*(anstate.next++)=code&0xff;
	anyl_result.collected++;
	anyl_result.newstatus=1;
	if (anyl_result.collected>=n) {
		anyl_result.collected=0;
		anstate.next=anstate.codebuf;
		anstate.new=0;
		return 1;
	} else if (_monitor) {
		// This really does nothing unless task has been supsended
		vTaskResume(_monitor);
	}
	return 0;
}

static u32 peak(u8 *buf, int n)
{
	int i;
	long low, high, curr;
	u8 *p=buf;

	low=high=get_from_buf(&p);
	for (i=1;i<n;++i) {
		curr=get_from_buf(&p);
		if (curr<low) low=curr;
		if (curr>high) high=curr;
	}
	return abs32(high-low);
}

static void anyl_peak(void)
{
	u32 m;
	int avgs=cstate.avgs;

	if (collect(avgs)) {
		setintresult(peak(anstate.codebuf,avgs));
	}
}

static void anyl_rmsn(void)
{
	float m;
	int avgs=cstate.avgs;

	if (collect(avgs)) {
		setresult(stddev(anstate.codebuf,avgs));
	}
}

static void anyl_avg(void)
{
	float m;
	int avgs=cstate.avgs;

	if (collect(avgs)) {
		setresult(avg(anstate.codebuf,avgs));
	}
}

static void anyl_raw(void)
{
	s32 code;

	ads_get(&code);		//from ads.c
	anyl_result.direct=code;
	anyl_result.newdirect=1;
	// This really does nothing unless task has been supsended
	//if (_directmonitor)
	//	vTaskResume(_directmonitor);
	setintresult(code);
}

static void anyl_task(void *p)
{	/* it doesn't appear that the following statement really does anything*/
	ads_set_monitor(this);	//from ads.c
	for(;;) {
		if (cstate.go) {
			if (ads_newdata()) {
				anyl_result.timeout=0;
				switch(cstate.mode) {
				case ANYL_MODE_RMSN:
					anyl_rmsn();
					break;
				case ANYL_MODE_PEAK:
					anyl_peak();
					break;
				case ANYL_MODE_AVG:
					anyl_avg();
					break;
				default:
					anyl_raw();
					break;
				}
				anyl_result.newstatus=1;
			} else {
				anyl_result.timeout=1;
				anyl_result.newstatus=1;
			}
		}
		vTaskDelay(ANYL_TIMEOUT);
	}
}

void anyl_set_mode(int m)
{
	if (m!=cstate.mode) {
		cstate.mode=m;
		cstate.changed=1;
	}
}

int anyl_get_mode(void)
{
	return cstate.mode;
}

void anyl_set_avgs(int a)
{
	if (a==cstate.avgs) return;
	cstate.avgs=a;
	cstate.changed=1;
}

u8 anyl_get_avgs(void)
{
	return cstate.avgs;
}

void anyl_go(void)
{
	if (cstate.go) return;
	cstate.go=1;
	ads_go();	// from ads.c
	anyl_restart();
}

void anyl_stop(void)
{
	if (!cstate.go) return;
	cstate.go=0;
	ads_stop();	// from ads.c
	#ifndef ETEST
	if (aparms.console) vTaskSuspend(this);
	#endif
	
}

void anyl_restart(void)
{
	cstate.changed=1;
	anyl_result.res.i=0;
}
/* What is the purpose for setting the _monitor...what is it's function? */
void anyl_set_monitor(xTaskHandle monitor)
{
	_monitor=monitor;
}

void anyl_set_directmonitor(xTaskHandle monitor)
{
	_directmonitor=monitor;
}
void anyl_init(int pri)
{
	_monitor=0;
	_directmonitor=0;
	cstate.mode=ANYL_MODE_RAW;
	cstate.changed=1;
	cstate.avgs=50;
	cstate.go=0;
	xTaskCreate(anyl_task,"anyl",90,0,pri,&this);
	anyl_task_handle=this;
	anyl_result.newresult=0;
	anyl_result.newdirect=0;
	anyl_result.status=ANYL_STAT_STOPPED;
	aparms.task=this;
}
