
#include "config.h"
#include "task.h"
#include "anyl.h"
#include "joystick.h"
#include "lcd.h"
#include "disp.h"
#include "flash.h"

extern xTaskHandle core_task_handle;
#define this core_task_handle
/* Screen count needs to match the number of functions defined in 
	static void (*screens[SCREEN_COUNT])(void) */
#define SCREEN_COUNT 8			// change to 7 without debug


#define SCALE_SCREEN_COUNT 11

//scale_parms_t sparms;
anyl_parms_t aparms;
state_t state;
reg_parms_t rparms;

u8 confmode;

char *unit_strings[]={"AU","ug","mg"," g","kg","oz","lb","st"};

void config_sc(void)
{
	/* Operation of ADC in a scale mode application.  Initially this will not
	be an option.  When later added this must include the proper calls for scale mode 
	so that the proper channels can be identified and setup. */


	//ads_set_ch(sparms.ch+1);

	//ads_set_speed(sparms.speed);

	//ads_set_pga(3);

	//if (sparms.disp_avgs>1)
		//anyl_set_mode(ANYL_MODE_AVG);
	//else
		//anyl_set_mode(ANYL_MODE_RAW);
	//anyl_set_avgs(sparms.disp_avgs);
}

void config_an(void)
{
	/* Config standard anylizer potion of the code.  This will require functions 
	to be called as match the ADC ability for PGA, SPEED, CHANNELs, etc.. Some of this
	info may be called from flash.....*/

	ads_set_ch(aparms.ch_p, aparms.ch_m);
	ads_set_speed(aparms.speed);
	ads_set_supply(aparms.supply);

	ads_set_pga(aparms.pga);
	ads_set_bo(aparms.burnout);
	ads_set_bias(aparms.bias);
	ads_set_vref(aparms.refsel);
	ads_set_iref(aparms.refin);
	ads_set_current(aparms.current);
	ads_set_idac0(aparms.idac0);
	ads_set_idac1(aparms.idac1);
	ads_set_gconfig(aparms.gconfig);
	ads_set_gio(aparms.gdir);
	ads_set_gdat(aparms.gpio);
	ads_set_mux(aparms.mux);
	anyl_set_mode(aparms.anyl_mode);
	anyl_set_avgs(aparms.avgs);
	ads_write();	
	
	//ads_cal();
}
/*
static int extclk_get(void)
{
	return sparms.extclk;
}

static int extclk_set(int extclk)
{
	sparms.extclk=extclk?1:0;
}
*/
static void screen_prev(void)
{
	if (op_mode==JS_MODE_AN) {
		if (state.ascreen==0) {
			state.ascreen=SCREEN_COUNT-1;
		} else {
			state.ascreen--;
		}
	} else {
		if (state.sscreen==0) {
			state.sscreen=SCALE_SCREEN_COUNT-1;
		} else {
			state.sscreen--;
		}
	}

	
}

static void screen_next(void)
{
		if (op_mode==JS_MODE_AN) {
		++state.ascreen;
		if (state.ascreen>=SCREEN_COUNT)
			state.ascreen=0;
	} else {
		++state.sscreen;
		if (state.sscreen>=SCALE_SCREEN_COUNT)
			state.sscreen=0;
	}


}
/* Check to see if this is true...blocking occurs on the JS task until something is in the queue*/
// if timeout != 0, this will return -2 after 0.4 sec with no key press
static int screen_key(int timeout, int feature)
{
	js_evt e;

	for(;;) {
		// what is the actual delay here?
		vTaskDelay(10);
		if (timeout&&!js_have())
			return -2;
		while (js_have()) {
			e=js_get();
			if (e.pressed) {
				if (js_pressed(JS_LEFT)&&!feature) {
						confmode=MODE_DISP;
					return -1;
				}
			}
			if (!e.pressed&&!e.held&&!feature) {
				switch(e.jssw) {
				case JS_UP:
					screen_next();
					return -1;
				case JS_DOWN:
					screen_prev();
					return -1;
				case JS_RIGHT:
					return JS_RIGHT;
				case JS_SELECT:
					return JS_SELECT;
				default:
					break;
				}
			}
			else if (!e.pressed&&!e.held) {
				switch(e.jssw) {
				case JS_UP:
					return JS_UP;
				case JS_DOWN:
					return JS_DOWN;
				case JS_RIGHT:
					screen_next();
					return -1;
				case JS_LEFT:
					screen_prev();
					return -1;
				case JS_SELECT:
					return JS_SELECT;
				default:
					break;
				}
			}
			
		}
	}
}
/* returns -2 if no key is pressed after 0.4sec if timeout is non-zero.
returns -1 if some kind of mode change happened
returns key value if RIGHT or SELECT is pressed
if timeout is 0 this blocks until a key is pressed
*/

static void scr_avgs(void)
{
	const int maxavgs=11;
	const u8 vals[]={1,2,4,8,10,16,32,50,64,100,128};
	int sel;
	u8 avgs;

	sel=0;
	if (op_mode==JS_MODE_SC)
		avgs=0;//avgs=sparms.disp_avgs;
	else
		avgs=aparms.avgs;
	while (vals[sel]!=avgs&&sel<maxavgs)
		++sel;
	lcd_showstr("AVGS=");
	for (;;) {
		int2asc((u32)vals[sel],3,1,dispbuf);
		lcd_showstr_pos(dispbuf,5);
		switch(screen_key(0,0)) {
			/* Currently the only mode working is right arrow.  This
			will increment through the list until the end then wrap
			to the beginning. */
		case JS_RIGHT:
			++sel;
			if (sel>=maxavgs) {
				sel=(op_mode==JS_MODE_SC)?0:1;
			}
			break;
		case JS_VALDN:
			--sel;
			if (sel<((op_mode==JS_MODE_SC)?0:1)) sel=maxavgs-1;
			break;
		case -1:
			//if (op_mode==JS_MODE_SC)
				//sparms.disp_avgs=vals[sel];
			//else
				aparms.avgs=vals[sel];
			return;
		default:
			break;
		}
	}
}
/* This section sets the reference voltage level for the ADS1248...which has internal or external reference */
static void scr_vref(void)
{
	char vrefdisp[5];
	float vref=disp_vref;
	int flash=1;
	int feature=1;

	lcd_showstr("REF");
	for(;;) {
		f2asc(vref,1,3,1,vrefdisp);
		if (flash) {
			switch(state.ascreen){
				case 1:
				vrefdisp[0]=' ';
				break;
				case 2:
				vrefdisp[2]=' ';
				break;
				case 3:
				vrefdisp[3]=' ';
				break;
				case 4:
				vrefdisp[4]=' ';
				break;
			}
		}
		lcd_showstr_pos(vrefdisp,4);
		switch(screen_key(1,feature)) {
		case JS_UP:
		
			switch(state.ascreen){
			case 1:
				vref+=1;
				break;
			case 2:
				vref+=0.1;
				break;
			case 3:
				vref+=0.01;
				break;
			case 4:
				vref+=0.001;
				break;
			}
				
			if (vref>5.0) vref=5.0;
			flash=0;
			break;
		case JS_DOWN:
			switch(state.ascreen){
			case 1:
				vref-=1;
				break;
			case 2:
				vref-=0.1;
				break;
			case 3:
				vref-=0.01;
				break;
			case 4:
				vref-=0.001;
				break;
			}
			if (vref<0.1) vref=0;
			flash=0;
			break;
		case -1:
			disp_vref=vref;
			return;
		case -2:
			flash=!flash;
			break;
		default:
			break;
		}
	}
}

void showunit(void)
{
	lcd_showstr("UNIT=");
	//lcd_showstr_pos(unit_strings[sparms.unit],6);
}

void incunit(void)
{
	//++sparms.unit;
	//if (sparms.unit>7) sparms.unit=1;
}

static void scr_unit(void)
{
	int sel;
/*
	sel=sparms.unit;
	lcd_showstr("UNIT=");
	for (;;) {
		lcd_showstr_pos(unit_strings[sel],6);
		switch(screen_key(0)) {
		case JS_VALUP:
			++sel;
			if (sel>7) sel=1;
			break;
		case JS_VALDN:
			--sel;
			if (sel<1) sel=7;
			break;
		case -1:
			sparms.unit=sel;
			return;
		default:
			break;
		}
	}*/
}

static void scr_speed(void)
{
	lcd_showstr("SPD=");
	for(;;) {
		//lcd_showstr_pos(sparms.speed?"FAST":"SLOW",4);
		switch(screen_key(0,0)) {
		case JS_VALUP:
		case JS_VALDN:
		//	sparms.speed=!sparms.speed;
			break;
		case -1:
			return;
		default:
			break;
		}
	}
}


static void scr_ch(void)
{
	lcd_showstr("CHANNEL");
	for(;;) {
		//lcd_showc('1'+sparms.ch,7);
		switch(screen_key(0,0)) {
		case JS_VALUP:
		case JS_VALDN:
			//sparms.ch=!sparms.ch;
			break;
		case -1:
			return;
		default:
			break;
		}
	}
}

/*
static void scr_calweight(void)
{
	char vrefdisp[4];
	float wt;
	int flash=1,sel;

	lcd_showstr("CW=");
	sel=sparms.calunit;
	if (wt>99.9) wt=99.9;
	for(;;) {
		wt=g2scale(sparms.w_fs,sel);
		if (wt>99.9) wt=99.9;
		f2asc(wt,2,1,2,vrefdisp);
		lcd_showstr_pos(vrefdisp,3);
		lcd_showstr_pos(unit_strings[sel],6);
		if (flash) {
			lcd_showc(' ',sparms.screen);
			if (sparms.screen>5)
				lcd_showc(' ',7);
		}
		switch(screen_key(1)) {
		case JS_VALUP:
			if (sparms.screen==3)
				wt+=10.0;
			else if (sparms.screen==4)
				wt+=1.0;
			else if (sparms.screen==5)
				wt+=0.1;
			else {
				if (++sel>7)
					sel=1;
			}
			if (sparms.screen<6) {
				sparms.w_fs=scale2g(wt,sel);
				if (wt>99.9) wt=0;
			}
			break;
		case JS_VALDN:
			if (sparms.screen==3)
				wt-=10.0;
			else if (sparms.screen==4)
				wt-=1.0;
			else if (sparms.screen==5)
				wt-=0.1;
			else {
				if (--sel<1)
					sel=7;
			}
			if (sparms.screen<6) {
				sparms.w_fs=scale2g(wt,sel);
				if (wt<0) wt=99.9;
			}
			break;
		case -1:
			sparms.calunit=sel;
			return;
		case -2:
			flash=!flash;
			break;
		default:
			break;
		}
	}
}
*/
static void scr_version(void)
{
	lcd_arrowset(1,0);	//up
	lcd_arrowset(3,1);	//down
	//lcd_arrowset(2,1);	//right
	lcd_arrowset(0,1);	//left	
	
	lcd_showstr("V" VERSION);
	
	while (screen_key(0,0)!=-1);
	
	lcd_arrowset(1,0);	//up
	lcd_arrowset(3,1);	//down
	lcd_arrowset(2,1);	//right
	lcd_arrowset(0,1);	//left	

}
/*
static void scr_clk(void)
{
	const char *clkstrs[3]={"INT","MCU","EXT"};
	//int sel=extclk_get();

	lcd_showstr("CLK=");
	for(;;) {
		lcd_showstr_pos((char *)(clkstrs[sel]),5);
		switch(screen_key(0)) {
		case -1:
	//		extclk_set(sel);
			return;
		case JS_VALUP:
			++sel;
			if (sel>2) sel=0;
			break;
		case JS_VALDN:
			--sel;
			if (sel<0) sel=2;
			break;
		default:
			break;
		}
	}
}
*/

void scr_debug(void);

static void scr_enter_debug(void)
{
	lcd_showstr("DEBUG?");
	for(;;) {
		switch(screen_key(0,0)) {
		case -1:
			return;
		case (JS_SELECT):
			scr_debug();
			return;
		case (JS_RIGHT):
			scr_debug();
			return;
		
		default:
			break;
		}
	}
}

static void scr_ocal(void)
{
	lcd_showstr("OCAL?");
	for(;;) {
		switch(screen_key(0,0)) {
		case -1:
			return;
		case (JS_SELECT):
			lcd_showstr("   OK");
			ads_cal();
			vTaskDelay(100);
			return;
		case (JS_RIGHT):
			lcd_showstr("   OK");
			ads_cal();
			vTaskDelay(100);
			return;
		default:
			break;
		}
	}
}

static void scr_save(void)
{
	lcd_showstr("SAVE?");
	for(;;) {
		switch(screen_key(0,0)) {
		case -1:
			return;
		case (JS_SELECT):
			lcd_showstr(" SAVING");
			ads_stop();
			flash_save_parms();
			ads_go();
			lcd_showstr("  DONE");
			vTaskDelay(100);
			return;
		case (JS_RIGHT):
			lcd_showstr(" SAVING");
			ads_stop();
			flash_save_parms();
			ads_go();
			lcd_showstr("  DONE");
			vTaskDelay(100);
			return;
		default:
			break;
		}
	}
}



static void scr_pdwn(void)
{
	lcd_showstr("PDWN?");
	for(;;) {
		switch(screen_key(0,0)) {
		case -1:
			return;
		case (JS_SELECT):
			lcd_showstr("POWER DN");
			ads_powerdown();
			js_block();
			js_waitrelease();
			ads_go();
			confmode=MODE_DISP;
			return;
		case (JS_RIGHT):
			lcd_showstr("POWER DN");
			ads_powerdown();
			js_block();
			js_waitrelease();
			ads_go();
			confmode=MODE_DISP;
			return;
		
		default:
			break;
		}
	}
}
/* Number of entries must match SCREEN_COUNT defined at top of file */
static void (*screens[SCREEN_COUNT])(void)={
	scr_avgs,
	scr_vref, // ones
	scr_vref, // tenths
	scr_vref, // hundredths
	scr_vref, // thousandths
	scr_pdwn,
	scr_ocal,
	//scr_save,		//- add of needing to save parameters to flash
	scr_version,
	//scr_enter_debug 	//- add if needing debugging
};

static void (*scale_screens[SCALE_SCREEN_COUNT])(void)={
	scr_unit,
	scr_avgs,
	scr_speed,
	//scr_calweight, // 3=tens
	//scr_calweight, // 4=ones
	//scr_calweight, // 5=tenths
	//scr_calweight, // 6=units


	scr_ch,

	scr_ocal,
	scr_save,
	scr_version
};

void config(void)
{
	js_evt e;
	anyl_stop();
	js_waitrelease();
	
	lcd_arrowset(1,0);	//up
	lcd_arrowset(3,1);	//down
	lcd_arrowset(2,1);	//right
	lcd_arrowset(0,1);	//left	

	
	confmode=MODE_CONF;
	for(;;) {
		e=js_get();
		if (!e.pressed&&!e.held) {
			if (!js_pressed(JS_DOWN)&&!js_pressed(JS_UP))
				break;
		}
	}
	for(;;) {
		if (op_mode==JS_MODE_AN) {
			screens[state.ascreen]();
		} else {
			scale_screens[state.sscreen]();
		}
		if (confmode!=MODE_CONF) break;
	}
	//anyl_go();
	lcd_arrowset(1,0);	//up
	lcd_arrowset(3,1);	//down
	lcd_arrowset(2,1);	//right
	//lcd_arrowset(0,1);	//left	



}
