
#include "common.h"
#include "uart.h"
#include "ads.h"
#include "anyl.h"
#include "config.h"
#include "joystick.h"
#include "lcd.h"
#include "i2c.h"

xTaskHandle console_task_handle;
#define this console_task_handle

#define echo(c) uart_txc(c)
#define print(c) uart_txstr(c)
/*
#define TEST_LIM_CH1_LO	8388000L
#define TEST_LIM_CH1_HI 8388607L
#define TEST_LIM_CH2_LO -1000L
#define TEST_LIM_CH2_HI 1000L
*/
static char lcase(char c)
{
	if (c>='A'&&c<='Z') return c+('a'-'A');
	else return c;
}

static char ucase(char c)
{
	if (c>='a'&&c<='z') return c-('a'-'A');
	else return c;
}

#ifndef ETEST
static const char *scr_speed_strings[10]={"   5","  10","  20","  40","  80"," 160"," 320"," 640","1000","2000"};
static const char *scr_gain_strings[8]={"  1","  2","  4","  8"," 16"," 32"," 64","128"};
static const char *scr_ch_strings[16]={"0","1","2","3","4","5","6","7","8","9","A","B","C","D","E","F"};
static const char *scr_bo_strings[4]={" OFF"," 0.5"," 2"," 10"};
static const char *scr_vref_strings[4]={" VREF0"," VREF1"," INTERNAL", " INTERNAL TO VREF0"};
static const char *scr_iref_strings[3]={" OFF"," ON"," FOLLOWS START"};
static const char *scr_current_strings[8]={" OFF"," 50uA"," 100uA"," 250uA"," 500uA"," 750uA"," 1000uA"," 1500uA"};
static const char *scr_idac_strings[11]={" AIN0"," AIN1"," AIN2"," AIN3"," AIN4"," AIN5"," AIN6"," AIN7"," IOUT1"," IOUT2"," OFF"};

#else
char eeprom1[34];
char eeprom2[18];
char eeprom3[18];
char eeprom4[18];
char eeprom5[18];


static const char scr_speed_strings[][6]={"   5","  10","  20","  40","  80"," 160"," 320"," 640","1000","2000"};
static const char scr_gain_strings[][4]={"  1","  2","  4","  8"," 16"," 32"," 64","128"};
static const char scr_ch_strings[][2]={"0","1","2","3","4","5","6","7","8","9","A","B","C","D","E","F"};
static const char scr_bo_strings[][5]={" OFF"," 0.5"," 2"," 10"};
static const char *scr_vref_strings[4]={" VREF0"," VREF1"," INTERNAL", "INTERNAL TO VREF0"};
static const char *scr_iref_strings[3]={" OFF"," ON"," FOLLOWS START"};
static const char scr_current_strings[][8]={" OFF"," 50uA"," 100uA"," 250uA"," 500uA"," 750uA"," 1000uA"," 1500uA"};
static const char scr_idac_strings[][6]={" AIN0"," AIN1"," AIN2"," AIN3"," AIN4"," AIN5"," AIN6"," AIN7"," IOUT1"," IOUT2"," OFF"};
#endif




static void set_datarate(char c)
{
	/* Calls made will depend on the ability of the converter to operate at 
	different speeds.... 'f'= fast and 's'= slow....the appropriate functions
	will need to be entered within the case statements */
	int spd = (int) c - 48;
	if (c==0) {
		print("speed=");
		print((char *)scr_speed_strings[ads_get_speed()]);
		print("\r\n");
		return;
	}
	
	if (spd==ads_get_speed()) return;
	
	if (spd>=0 && spd<=9) {
		
		if (op_mode==JS_MODE_AN) {
			aparms.speed=spd;
			config_an();
		} else {
			//sparms.speed=spd;
			config_sc();
		}
	if (!core_get_task()) anyl_go();
	return;
	}
	
	
	print("0,1,...9\r\n");
		
	
	
	
		
}

static void set_pga(char c)
{
	/* Calls made will depend on the ability of the converter to operate at 
	different gains....the appropriate functions
	will need to be entered within the case statements */
	
	int spd = (int) c - 48;
	if (c==0) {
		print("pga=");
		print((char *)scr_gain_strings[ads_get_pga()]);
		print("\r\n");
		return;
	}
	
	if (spd==ads_get_pga()) return;
	
	if (spd>=0 && spd<=7) {
		
		if (op_mode==JS_MODE_AN) {
			aparms.pga=spd;
			config_an();
		} else {
			//sparms.pga=spd;
			config_sc();
		}
	if (!core_get_task()) anyl_go();
	return;
	}
	
	
	print("0,1,...7\r\n");
		
	
}

static void set_mux(char c)
{
	/* Calls made will depend on the ability of the converter to operate at 
	different mux readings....the appropriate functions
	will need to be entered within the case statements */
	
	int mux = (int) c - 48;
	if (c==0) {
		print("mux=");
		print((char *)scr_ch_strings[ads_get_mux()]);
		print("\r\n");
		return;
	}
	
	if (mux==ads_get_mux()) return;
	
	if (mux>=0 && mux<=7) {
		
		if (op_mode==JS_MODE_AN) {
			aparms.mux=mux;
			config_an();
		} else {
			//sparms.pga=spd;
			config_sc();
		}
	if (!core_get_task()) anyl_go();
	return;
	}
	
	
	print("0,1,...7\r\n");
		
	
}


static void set_buffer(char c)
{
	/* Calls made will depend on the ability of the converter to have a buffer....
	'e'= enable and 'd'= disable....the appropriate functions
	will need to be entered within the case statements */
	
	switch(c) {
	case 'e':
		//ads1226_set_buffer(1);
		break;
	case 'd':
		//ads1226_set_buffer(0);
		break;
	case 0:
		//print("buffer=");
		//if (ads1226_get_buffer())
		//	print("enabled\r\n");
		//else
		//	print("disabled\r\n");
		break;
	default:
		print("?e=enable, d=disable\r\n");
		break;
	}
}

static void set_burnout(char c)
{
	/* Calls made will depend on the ability of the converter to have a burnout current source....
	0,1, 2 or 3 are the correct responses....the appropriate functions
	will need to be entered within the case statements */
	
	int bo = (int) c - 48;
	if (c==0) {
		print("burnout=");
		print((char *)scr_bo_strings[ads_get_bo()]);
		print("\r\n");
		return;
	}
	
	if (bo==ads_get_bo()) return;
	
	if (bo>=0 && bo<=3) {
		
		if (op_mode==JS_MODE_AN) {
			aparms.burnout=bo;
			config_an();
		} else {
			//sparms.burnout=bo;
			config_sc();
		}
	if (!core_get_task()) anyl_go();
	return;
	}
	print("0,1,2 or 3\r\n");
}
static void set_vref(char c)
{
	/* Calls made will depend on the ability of the converter to have various reference sources....
	0,1, or 2 are the correct responses....the appropriate functions
	will need to be entered within the case statements */
	
	int vr = (int) c - 48;
	if (vr==1){
		print("0,2 or 3\r\n");
		return;
	}
	if (c==0) {
		print("vref=");
		print((char *)scr_vref_strings[ads_get_vref()]);
		print("\r\n");
		return;
	}
	
	if (vr==ads_get_vref()) return;
	
	if (vr>=0 && vr<=3) {
		
		if (op_mode==JS_MODE_AN) {
			aparms.refsel=vr;
			config_an();
		} else {
			//sparms.burnout=bo;
			config_sc();
		}
	if (!core_get_task()) anyl_go();
	return;
	}
	print("0,2 or 3\r\n");
}
static void set_iref(char c)
{
	/* Calls made will depend on the ability of the converter to have an internal refeference source....
	0,1,or 2 are the correct responses....the appropriate functions
	will need to be entered within the case statements */
	
	int ir = (int) c - 48;
	if (c==0) {
		print("iref=");
		print((char *)scr_iref_strings[ads_get_iref()]);
		print("\r\n");
		return;
	}
	
	if (ir==ads_get_iref()) return;
	
	if (ir>=0 && ir<=2) {
		
		if (op_mode==JS_MODE_AN) {
			aparms.refin=ir;
			config_an();
		} else {
			//sparms.burnout=bo;
			config_sc();
		}
	if (!core_get_task()) anyl_go();
	return;
	}
	print("0,1,or 2\r\n");
}

static void set_current(char c)
{
	/* Calls made will depend on the ability of the converter to have an internal current source....
	0,1, through 7 are the correct responses....the appropriate functions
	will need to be entered within the case statements */
	
	int ir = (int) c - 48;
	if (c==0) {
		print("current=");
		print((char *)scr_current_strings[ads_get_current()]);
		print("\r\n");
		return;
	}
	
	if (ir==ads_get_current()) return;
	
	if (ir>=0 && ir<=7) {
		
		if (op_mode==JS_MODE_AN) {
			aparms.current=ir;
			config_an();
		} else {
			//sparms.burnout=bo;
			config_sc();
		}
	if (!core_get_task()) anyl_go();
	return;
	}
	print("0,1,through 7\r\n");
}
static void set_idac(char c, int i)
{
	/* Calls made will depend on the ability of the converter to have an internal current source....
	0,1,through A hex are the correct responses....the appropriate functions
	will need to be entered within the case statements */
	
	int ir = (int) c - 48;
	if (c==0) {
		if (i) {
			print("idac1=");
			print((char *)scr_idac_strings[ads_get_idac1()]);
		}else {
			print("idac0=");
			print((char *)scr_idac_strings[ads_get_idac0()]);
		}
		print("\r\n");
		return;
	}
	
	// Using a ADS1247, can't allow upper channels or IDAC outs
	if  (ir>=4 && ir<=9){
		print("0,1,...3 or ':' for 10\r\n");
		return;
	
	}
	if (i) {
		if (ir==ads_get_idac1()) return;
	} else {
		if (ir==ads_get_idac0()) return;
	}
	if (ir>=0 && ir<=10) {
		
		if (op_mode==JS_MODE_AN) {
			if (i) aparms.idac1=ir;
			else aparms.idac0=ir;
			config_an();
		} else {
			//sparms.burnout=bo;
			config_sc();
		}
	if (!core_get_task()) anyl_go();
	return;
	}
	print("0,1,...3 or ':' for 10\r\n");
}

static void print_byte(u8 b_c)
{
		print((char *)scr_ch_strings[b_c>>4]);
		print((char *)scr_ch_strings[b_c&0x0f]);
		
}

static int hex2int(int cbyte)
{
	if (cbyte>=0 && cbyte<=22){
		if (cbyte>=17 && cbyte<=22){
			cbyte = cbyte-7;
		}
		
	}
	return cbyte;
}

static void set_bias(char high, char low)
{
	/* Calls made will depend on the ability of the converter to have a bias voltage setting....
	0,1, 2 through 7 are the correct responses....the appropriate functions
	will need to be entered within the case statements */
	
	u8 b_c;
		
	//int hbias = (int) high - 48;
	
	int hbias = 0;
	int lbias = (int) ucase(low) - 48;
	
	if (high==0) {
		b_c=ads_get_bias();
		print("bias=");
		print_byte(b_c);
		print("\r\n");
		return;
	}
	
	//hbias=hex2int(hbias);
	lbias=hex2int(lbias);
	
		
	if ((hbias>=0 && hbias<=15) && (lbias>=0 && lbias<=15)){
		if (op_mode==JS_MODE_AN) {
			aparms.bias= ((u8)(hbias<<4)|(u8)lbias);
			config_an();
		} else {
			
			config_sc();
		}
		
	if (!core_get_task()) anyl_go();
	return;
	}
	
	print("00 through 0f hex representation of a byte is acceptable\r\n");
}

static void set_gconfig(char high, char low)
{
	/* Calls made will depend on the ability of the converter to have a gpio setting....
	0,1, 2 through 7 are the correct responses....the appropriate functions
	will need to be entered within the case statements along with whether the setting
	is high or low. */
	
	u8 b_c;
		
	//int hbias = (int) high - 48;
	int hbias = 0;
	int lbias = (int) ucase(low) - 48;
	
	if (high==0) {
		b_c=ads_get_gconfig();
		print("gpio config=");
		print_byte(b_c);
		print("\r\n");
		return;
	}
	
	//hbias=hex2int(hbias);
	lbias=hex2int(lbias);
	
		
	if ((hbias>=0 && hbias<=15) && (lbias>=0 && lbias<=15)){
		if (op_mode==JS_MODE_AN) {
			aparms.gconfig= ((u8)(hbias<<4)|(u8)lbias);
			config_an();
		} else {
			
			config_sc();
		}
		
	if (!core_get_task()) anyl_go();
	return;
	}
	
	print("00 through 0f hex representation of a byte is acceptable\r\n");
}
static void set_gio(char high, char low)
{
	/* Calls made will depend on the ability of the converter to have a gpio setting....
	0,1, 2 through 7 are the correct responses....the appropriate functions
	will need to be entered within the case statements along with whether the setting
	is high or low. */
	
	u8 b_c;
		
	//int hbias = (int) high - 48;
	int hbias = 0;
	int lbias = (int) ucase(low) - 48;
	
	if (high==0) {
		b_c=ads_get_gio();
		print("gpio dir=");
		print_byte(b_c);
		print("\r\n");
		return;
	}
	
	//hbias=hex2int(hbias);
	lbias=hex2int(lbias);
	
		
	if ((hbias>=0 && hbias<=15) && (lbias>=0 && lbias<=15)){
		if (op_mode==JS_MODE_AN) {
			aparms.gdir= ((u8)(hbias<<4)|(u8)lbias);
			config_an();
		} else {
			
			config_sc();
		}
		
	if (!core_get_task()) anyl_go();
	return;
	}
	
	print("00 through 0f hex representation of a byte is acceptable\r\n");
	
}

static void set_gdat(char high, char low)
{
	/* Calls made will depend on the ability of the converter to have a gpio setting....
	0,1, 2 through 7 are the correct responses....the appropriate functions
	will need to be entered within the case statements along with whether the setting
	is high or low. */
	
	u8 b_c;
		
	//int hbias = (int) high - 48;
	int hbias = 0;
	int lbias = (int) ucase(low) - 48;
	
	if (high==0) {
		b_c=ads_get_gdat();
		print("gpio write=");
		print_byte(b_c);
		print("\r\n");
		return;
	}
	
	//hbias=hex2int(hbias);
	lbias=hex2int(lbias);
	
		
	if ((hbias>=0 && hbias<=15) && (lbias>=0 && lbias<=15)){
		if (op_mode==JS_MODE_AN) {
			rparms.gpiodat=aparms.gpio= ((u8)(hbias<<4)|(u8)lbias);
			config_an();
		} else {
			
			config_sc();
		}
		
	if (!core_get_task()) anyl_go();
	return;
	}
	
	print("00 through 0f hex representation of a byte is acceptable\r\n");
	
}

static void get_gdat(void)
{
	/* Calls made will depend on the ability of the converter to have a gpio setting....
	0,1, 2 through 7 are the correct responses....the appropriate functions
	will need to be entered within the case statements along with whether the setting
	is high or low. */
	
	u8 b_c=ads_get_gin();
	print("gpio read=");
	print_byte(b_c);
	print("\r\n");
	return;

	
	
}


static void set_ch(char c, int i)
{
	/* Calls made will depend on the ability of the converter to have multiple
	channels....
	'0'= CH1 and '1'= CH2 and 't'= temp....the appropriate functions
	will need to be entered within the case statements */
	
	int spd = (int) c - 48;
	if (c==0) {
		print("MUX+=");
		print((char *) scr_ch_strings[ads_get_ch(0)]);
		print("      MUX-=");
		print((char *) scr_ch_strings[ads_get_ch(1)]);
		print("\r\n");
		return;
	}
	
	if (i==0){
		if (c==ads_get_ch(0)) return;
	} else if (c==ads_get_ch(1)) return;
	
	if (spd>=0 && spd<=3) {
		
		
		if (op_mode==JS_MODE_AN) {
			if (i==0){
			aparms.ch_p=spd;
			} else if (i==1) {
			aparms.ch_m=spd;
			}
			config_an();
		} else {
			//sparms.pga=spd;
			config_sc();
		}
	if (!core_get_task()) anyl_go();
	return;
	}
	
	
	print("0,1,...3\r\n");
		
}

static void set_su(char c)
{
		
	int spd = (int) c - 48;
	if (c==0) {
		print("SUPPLY=");
		print((char *) scr_ch_strings[ads_get_supply()]);
		print("\r\n");
		return;
	}
	
	
	if (c==ads_get_supply()) return;
	
	
	if (spd>=0 && spd<=1) {
		
		
		if (op_mode==JS_MODE_AN) {
			
			aparms.supply=spd;
			
			config_an();
		} else {
			//sparms.pga=spd;
			config_sc();
		}
	if (!core_get_task()) anyl_go();
	return;
	}
	
	
	print("0 or 1\r\n");
		
}

static void set_pwrdn(char c)
{
		
	int spd = (int) c - 48;
	if (c==0) {
		print("POWER DOWN=");
		print((char *) scr_ch_strings[ads_get_pwrdn()]);
		print("\r\n");
		return;
	}
	
	
	if (c==ads_get_pwrdn()) return;
	
	
	if (spd>=0 && spd<=1) {
		
		
		if (op_mode==JS_MODE_AN) {
			
			aparms.pwrdn=spd;
			ads_set_pwrdn(aparms.pwrdn);
			if (spd) {
				config_an();
				
			}
		} else {
			//sparms.pga=spd;
			//config_sc();
		}
	if (!core_get_task()) anyl_go();
	return;
	}
	
	
	print("0 or 1\r\n");
		
}
static void set_full(char chigh1, char chigh0, char cmid1, char cmid0, char clow1, char clow0)
{
	
	int hcal1 = (int) ucase(chigh1) - 48;
	int hcal0 = (int) ucase(chigh0) - 48;
	int mcal1 = (int) ucase(cmid1) - 48;
	int mcal0 = (int) ucase(cmid0) - 48;
	int lcal1 = (int) ucase(clow1) - 48;
	int lcal0 = (int) ucase(clow0) - 48;
	
	if (chigh1==0) {
		ads_reg_read();
		print("FS Cal Coeff=");
		print_byte(rparms.fsc2);
		print_byte(rparms.fsc1);
		print_byte(rparms.fsc0);
		print("\r\n");
		return;
	}
	
	hcal1=hex2int(hcal1);
	hcal0=hex2int(hcal0);
	mcal1=hex2int(mcal1);
	mcal0=hex2int(mcal0);
	lcal1=hex2int(lcal1);
	lcal0=hex2int(lcal0);
	
		
	if ((hcal1>=0 && hcal1<=15) && (hcal0>=0 && hcal0<=15) && 
		(mcal1>=0 && mcal1<=15) && (mcal0>=0 && mcal0<=15) && 
		(lcal1>=0 && lcal1<=15) && (lcal0>=0 && lcal0<=15)){
		if (op_mode==JS_MODE_AN) {
			rparms.fsc2 = ((u8)(hcal1<<4)|(u8)hcal0);
			rparms.fsc1 = ((u8)(mcal1<<4)|(u8)mcal0);
			rparms.fsc0 = ((u8)(lcal1<<4)|(u8)lcal0);
			ads_write_full();
		} else {
			
			//config_sc();
		}
		
	if (!core_get_task()) anyl_go();
	return;
	}
	
	print("00 through ff hex representation of a byte for a 24-bit word is acceptable\r\n");
	

}

static void set_off(char chigh1, char chigh0, char cmid1, char cmid0, char clow1, char clow0)
{
	int hcal1 = (int) ucase(chigh1) - 48;
	int hcal0 = (int) ucase(chigh0) - 48;
	int mcal1 = (int) ucase(cmid1) - 48;
	int mcal0 = (int) ucase(cmid0) - 48;
	int lcal1 = (int) ucase(clow1) - 48;
	int lcal0 = (int) ucase(clow0) - 48;
	
	if (chigh1==0) {
		ads_reg_read();
		print("Offset Cal Coeff=");
		print_byte(rparms.ofc2);
		print_byte(rparms.ofc1);
		print_byte(rparms.ofc0);
		print("\r\n");
		return;
	}
	
	hcal1=hex2int(hcal1);
	hcal0=hex2int(hcal0);
	mcal1=hex2int(mcal1);
	mcal0=hex2int(mcal0);
	lcal1=hex2int(lcal1);
	lcal0=hex2int(lcal0);
	
	if ((hcal1>=0 && hcal1<=15) && (hcal0>=0 && hcal0<=15) && 
		(mcal1>=0 && mcal1<=15) && (mcal0>=0 && mcal0<=15) && 
		(lcal1>=0 && lcal1<=15) && (lcal0>=0 && lcal0<=15)){
		if (op_mode==JS_MODE_AN) {
			rparms.ofc2 = ((u8)(hcal1<<4)|(u8)hcal0);
			rparms.ofc1 = ((u8)(mcal1<<4)|(u8)mcal0);
			rparms.ofc0 = ((u8)(lcal1<<4)|(u8)lcal0);
			ads_write_off();
		} else {
			
			//config_sc();
		}
		
	if (!core_get_task()) anyl_go();
	return;
	}
	
	print("OO through ff hex representation of a byte for a 24-bit word is acceptable\r\n");
	

}

static void set_dac(char chigh1, char chigh0, char cmid1, char cmid0, char clow)
{
	s32 ndac=524287;
	s32 temp;
	int hdac1 = (int) ucase(chigh1) - 48;
	int hdac0 = (int) ucase(chigh0) - 48;
	int mdac1 = (int) ucase(cmid1) - 48;
	int mdac0 = (int) ucase(cmid0) - 48;
	int ldac = (int) ucase(clow) - 48;
	
	if (chigh1==0) {
		dac_output(ndac);
		DACOUTL();
		print("ok\r\n");
		return;
	}
	
	hdac1=hex2int(hdac1);
	hdac0=hex2int(hdac0);
	mdac1=hex2int(mdac1);
	mdac0=hex2int(mdac0);
	ldac=hex2int(ldac);
	
	if ((hdac1>=0 && hdac1<=15) && (hdac0>=0 && hdac0<=15) && 
		(mdac1>=0 && mdac1<=15) && (mdac0>=0 && mdac0<=15) && 
		(ldac>=0 && ldac<=15)) {
		
			temp = hdac1;
			ndac += (temp<<16);
			temp = hdac0;
			ndac += (temp<<12);
			temp = mdac1;
			ndac += (temp<<8);
			temp += mdac0;
			ndac += (temp<<4);
			ndac += ldac;
		
		
	if (ndac!=524287) {
		DACOUTH();
	} else DACOUTL();
	dac_output(ndac);
	print("ok\r\n");
	return;
		}
	
	print("00 through ff hex representation of a byte for a 20-bit word is acceptable\r\n");
	
}


/* Check and see if this function can be deleted */
static void get_ads(s32 *d)
{
	/* This function assigns the global data to the local variable for display. 
	This function is no longer used */
	
	while(!anyl_result.newdirect) {
		vTaskDelay(1);
	}
	anyl_result.newdirect=0;
	*d=anyl_result.direct;
}

static void printhex(s32 *d)
{
		/* Creates a buffer and converts hex data byte to ASCII characters
		in the buffer string....then displays it on the console along with
		a CR and LF*/
	char buf[7];
	
	hex2asc(d,6,2,buf);
	print(buf);
	uart_txc('\r');
	uart_txc('\n');
}

static void eeprom_read(void)
{
	char cbuf[32];
	u8 rbuf[2];
	rbuf[0]=0x00;
	rbuf[1]=0x00;
	int i=0;
	I2CErr err=1;
	
	do{
	err=i2cWrite(0xa0,rbuf,2);
	} while (err!=I2C_ERR_OK);
	err=1;
	do{
	err=i2cRead(0xa1,cbuf,32);
	} while (err!=I2C_ERR_OK);
	err=1;
	
	for (i=0;i<32;i++) {
		if (cbuf[i]!=0) uart_txc(cbuf[i]);
		else break;
	}
	uart_txc('|');
	rbuf[1]=0x20;
	do{
	err=i2cWrite(0xa0,rbuf,2);
	} while (err!=I2C_ERR_OK);
	err=1;
	do{
	err=i2cRead(0xa1,cbuf,16);
	} while (err!=I2C_ERR_OK);
	err=1;
	
	
	for (i=0;i<16;i++) {
		if (cbuf[i]!=0) uart_txc(cbuf[i]);
		else break;
	}
	uart_txc('|');
	rbuf[1]=0x30;
	
	do{
	err=i2cWrite(0xa0,rbuf,2);
	} while (err!=I2C_ERR_OK);
	err=1;
	do{
	err=i2cRead(0xa1,cbuf,16);
	} while (err!=I2C_ERR_OK);
	err=1;
	
	for (i=0;i<16;i++) {
		if (cbuf[i]!=0) uart_txc(cbuf[i]);
		else break;
	}
	uart_txc('|');
	rbuf[1]=0x40;
	
	do{
	err=i2cWrite(0xa0,rbuf,2);
	} while (err!=I2C_ERR_OK);
	err=1;
	do{
	err=i2cRead(0xa1,cbuf,16);
	} while (err!=I2C_ERR_OK);
	err=1;
	
	for (i=0;i<16;i++) {
		if (cbuf[i]!=0) uart_txc(cbuf[i]);
		else break;
	}
	uart_txc('|');
	rbuf[1]=0x50;
	
	do{
	err=i2cWrite(0xa0,rbuf,2);
	} while (err!=I2C_ERR_OK);
	err=1;
	do{
	err=i2cRead(0xa1,cbuf,16);
	} while (err!=I2C_ERR_OK);
	err=1;
	
	for (i=0;i<16;i++) {
		if (cbuf[i]!=0) uart_txc(cbuf[i]);
		else break;
	}
	
	uart_txc('\r');
	uart_txc('\n');
}

#ifndef ETEST
static void eeprom_write(char *c, int w )
{
	print("Option not available.....\r\n");
}
#else
static void eeprom_write(char *c, int w )
{
	
	u8 rbuf[2];
	
	int i=0;
	int j=0;
	I2CErr err=1;
	
	switch (c[2]) {
		case '1':
			eeprom1[0]=0x00;
			eeprom1[1]=0x00;
			
		for (i=2;i<34;i++) {
			eeprom1[i]=c[i+1];
			if (c[i+1]==0) {	//pad remaining with 0xff 
				
				for (j=i+2;j<34;j++){
					eeprom1[j]=0xff;
				}
				
				break;
			}
			
		}
		
			break;
		case '2':
			eeprom2[0]=0x00;
			eeprom2[1]=0x20;
			
		for (i=2;i<18;i++) {
			eeprom2[i]=c[i+1];
			if (c[i+1]==0) {	//pad remaining with 0xff 
				for (j=i+2;j<18;j++){
					eeprom2[j]=0xff;
				}
				
				break;
			}
			
		}
		
			break;
		case '3':
			eeprom3[0]=0x00;
			eeprom3[1]=0x30;
			
		for (i=2;i<18;i++) {
			eeprom3[i]=c[i+1];
			if (c[i+1]==0) {	//pad remaining with 0xff 
				for (j=i+2;j<18;j++){
					eeprom3[j]=0xff;
				}
				
				break;
			}
			
		}
			break;
		case '4':
			eeprom4[0]=0x00;
			eeprom4[1]=0x40;
			
		for (i=2;i<18;i++) {
			eeprom4[i]=c[i+1];
			if (c[i+1]==0) {	//pad remaining with 0xff 
				for (j=i+2;j<18;j++){
					eeprom4[j]=0xff;
				}
				
				break;
			}
			
		}
			break;
		case '5':
			eeprom5[0]=0x00;
			eeprom5[1]=0x50;
			
		for (i=2;i<18;i++) {
			eeprom5[i]=c[i+1];
			if (c[i+1]==0) {	//pad remaining with 0xff 
				for (j=i+2;j<18;j++){
					eeprom5[j]=0xff;
				}
				
				break;
			}
			
		}
			break;
		default:
			break;
		
	}
	
	if (w){
	//write eeprom1 array
	do{
	err=i2cWrite(0xa0,eeprom1,34);
	} while (err!=I2C_ERR_OK);
	err=1;
	
	//write eeprom2 array
	do{
	err=i2cWrite(0xa0,eeprom2,18);
	} while (err!=I2C_ERR_OK);
	err=1;
	
	//write eeprom3 array
	do{
	err=i2cWrite(0xa0,eeprom3,18);
	} while (err!=I2C_ERR_OK);
	err=1;
	
	//write eeprom4 array
	do{
	err=i2cWrite(0xa0,eeprom4,18);
	} while (err!=I2C_ERR_OK);
	err=1;
	
	//write eeprom5 array
	do{
	err=i2cWrite(0xa0,eeprom5,18);
	} while (err!=I2C_ERR_OK);
	err=1;
	
	}
}
#endif


static void data(char *s)
{
	s32 code;
	if (core_get_task()) {
		vTaskResume(aparms.task);
		anyl_go();
	}
	get_ads(&code);
	printhex(&code);
	if (core_get_task()) {
		anyl_stop();
		lcd_showstr("PC CONS");
	}
}

static void stream(void)
{
	s32 code;
/* To adequately stream data to the console at high data rates, the flag directmonitor is set.
	This means that the data will be spit out the console in the ISR instead of collected and redistributed from the buffer */
	// If analyzer task was suspended need to resume
	if (core_get_task()) {
		vTaskResume(aparms.task);
		anyl_go();
	}
	ads_set_directmonitor(this);
	do {
		vTaskDelay(10);
		lcd_showstr("STREAM");
		//get_ads(&code);
		//printhex(&code);
	} while (!uart_have());
	ads_set_directmonitor(0);
	uart_get();
	if (core_get_task()) {
		anyl_stop();
		lcd_showstr("PC CONS");
	}
	
}

/* This has not yet been completed */
static void show_parms(void)
{
	int i, c_t;
	ads_reg_read();
	u8 b_c=ads_get_bias();
	
	/* Displays on the console a list of current parameter settings.....
	This will need to change with regard to which parameters the ADC has and
	are available to set.....most functions assume a two state variable and uses
	a logical to determine actual printed value.  Statement ends with a CRLF */
	uart_txc('P');
	uart_txc(ads_get_pga()+'0');
	uart_txc('|');
	uart_txc('R');
	uart_txc(ads_get_speed()+'0');
	uart_txc('|');
	uart_txc('C');
	uart_txc('P');
	uart_txc(ads_get_ch(0)+'0');
	uart_txc('|');
	uart_txc('C');
	uart_txc('M');
	uart_txc(ads_get_ch(1)+'0');
	uart_txc('|');
	uart_txc('N');
	uart_txc(ads_get_bo()+'0');
	uart_txc('|');
	uart_txc('O');
	//for (i=7;i>-1;i--) { 
	//	c_t=(b_c>>i) & 0x01;	// scan through the list and show whether on or off
	//	if (c_t) uart_txc('1');
	//	else uart_txc('0');
	//	}
	print_byte(b_c);
	uart_txc('|');
	uart_txc('W');
	uart_txc(ads_get_vref()+'0');
	uart_txc('|');
	uart_txc('E');
	uart_txc(ads_get_iref()+'0');
	uart_txc('|');
	uart_txc('I');
	uart_txc(ads_get_current()+'0');
	uart_txc('|');
	uart_txc('X');
	uart_txc(ads_get_idac0()+'0');
	uart_txc('|');
	uart_txc('Y');
	uart_txc(ads_get_idac1()+'0');
	uart_txc('|');
	uart_txc('M');
	uart_txc(ads_get_mux()+'0');
	uart_txc('|');
	uart_txc('F');
	b_c=ads_get_gconfig();
	print_byte(b_c);
	uart_txc('|');
	//for (i=0;i<8;i++) { 
	//	c_t=(b_c>>i) & 0x01;	// scan through the list and show whether on or off
	//	if (c_t) uart_txc('1');
	//	else uart_txc('0');
	//	}
	uart_txc('U');
	b_c=ads_get_gio();
	print_byte(b_c);
	uart_txc('|');
	//for (i=0;i<8;i++) { 
	//	c_t=(b_c>>i) & 0x01;	// scan through the list and show whether on or off
	//	if (c_t) uart_txc('1');
	//	else uart_txc('0');
	//	}
		
	
		
	uart_txc('J');
	b_c=ads_get_gin();
	print_byte(b_c);
	uart_txc('|');
	uart_txc('L');
	print_byte(rparms.ofc2);
	print_byte(rparms.ofc1);
	print_byte(rparms.ofc0);
	uart_txc('|');
	uart_txc('H');
	print_byte(rparms.fsc2);
	print_byte(rparms.fsc1);
	print_byte(rparms.fsc0);
	uart_txc('|');
	//for (i=0;i<8;i++) { 
	//	c_t=(b_c>>i) & 0x01;	// scan through the list and show whether on or off
	//	if (c_t) uart_txc('1');
	//	else uart_txc('0');
	//	}
	//uart_txc('B');
	//uart_txc(ads1226_get_buffer()?'E':'D');
	//uart_txc('C');
	//uart_txc(ads1226_get_ch()+'0');
	uart_txc('I');
	uart_txc('D');
	print((char *)scr_ch_strings[rparms.idac0>>4]);
	uart_txc('|');
	uart_txc('S');
	uart_txc('U');
	uart_txc(ads_get_supply()+'0');
	
	uart_txc('|');
	uart_txc('P');
	uart_txc('N');
	uart_txc(ads_get_pwrdn()+'0');
	
	uart_txc('\r');
	uart_txc('\n');
}

static void show_version(void)
{
	/* VERSION is the value set during 'make' and is entered in the makefile...
	Edit the actual value to current board.  It is possible to use the default of 
	MMB3 and look for EEPROM data and transfer that value instead....*/
	
	//print("MMB3 " VERSION " (c)2008 Texas Instruments\r\n");
	#ifndef ADS1147
	//print("MMB3 " VERSION " (c)2008 Texas Instruments\r\n");
	#ifndef ETEST
	print("ADS1247EVM " VERSION " (c)2009 Texas Instruments\r\n");
	#else
	print("ADS1247EVM TEST " VERSION " (c)2009 Texas Instruments\r\n");
	#endif
	#else
	#ifndef ETEST
	print("ADS1147EVM " VERSION " (c)2009 Texas Instruments\r\n");
	#else
	print("ADS1147EVM TEST " VERSION " (c)2009 Texas Instruments\r\n");
	#endif
	#endif
}

static void credits(void)
{
	/* Credit info is printed to console.  This info must change for each
	set of firmware developed. */
	
	print("Hardware: Hiro Ishii, Rick Downs\r\n"
	      "Layout & tech: Bob Benjamin\r\n"
	      "Firmware: Rick Downs, Bob Benjamin\r\n"
	      "\r\n"
	      "(Boredom's not a burden anyone should bear)\r\n");
}

static int check_uart(void)
{
	vTaskDelay(50);
	if (!(P2IN&0x20)) return 0;
	return 1;
}
/*
static void adsevm_test(void)
{
	/* This is a console activated test using screen console to display
	results.  This really should be under the file 'test.c', but does give 
	another test type option that does not need full test functionality of the 
	MMB3.  This test can run and display results on the console instead of the LCD.
	The test does simple noise tests to see if ADC is within limits...a simple
	go no-go test. */
	
/*	s32 code;
	float enob; 
	char s[10];
	u32 i;
	
	anyl_set_mode(ANYL_MODE_RAW);
	
	print("ADS1226EVM Test\r\n");
	print("Set VRN to 0V\r\n");
	print("Set VRP to 2.5V\r\n");
	print("Set INP1 to VRP\r\n");
	print("Set INN1 to 0V\r\n");
	print("Set INP2 to 0V\r\n");
	print("Set INN2 to 0V\r\n");

	set_buffer('d');	//disable buffer
	set_ch('0',0);		//AIN1
	print("Press any key to continue\r\n");
	while(!uart_have());
	
	get_ads(&code);
	printhex(&code);
	
	if (code<TEST_LIM_CH1_LO||code>TEST_LIM_CH1_HI){
		print("FAIL Channel 1\r\n");
		printhex(&code);
		return;
	}
	else{
		set_ch('1',0);		//AIN2
		get_ads(&code);
		if ((code<TEST_LIM_CH2_LO)||(code>TEST_LIM_CH2_HI)){
			print("FAIL Channel 2\r\n");
			printhex(&code);
			return;
		}
	}

	anyl_set_mode(ANYL_MODE_RMSN);
	anyl_set_avgs(50);
	aparms.unit=ANYL_UNIT_ENOB;

	print ("Check LCD for ENOB reading... if greater than 19, PASS, if less, FAIL due to NOISE");

}
*/
static void console(void *p)
{
	/* This code section establishes what is seenn as the console prompt.  It could 
	be a value read in from the EEPROM as to what is displayed....and could default to MMB3.
	Depending on the options of the part, various settings could be used as options
		'r' = datarate
		'p' = pga
		'b' = buffer
		'cp' = + input channel
		'cm' = - input channel
		'v' = version
		's' = stream data
		'd' = get one data byte (word)
		'q' = query parameters and display
		'k' = calibrate
		'a' = credits
		't' = console test mode
		'n' = burn out current source
		'o' = bias output voltage connection and defaults low 'l' or can tell to go 'h' high
			format is 'o1h' to turn on bias to channel 1 and to turn off either 'o1' or 'o1l'
		'w' = reference select, i.e. internal or external
		'e' = internal voltage reference on or off
		'ee1'= write contents to eeprom1 buffer
		'ee2'= write contents to eeprom1 buffer
		'ee3'= write contents to eeprom1 buffer
		'ee4'= write contents to eeprom1 buffer
		'ee5'= write contents to eeprom1 buffer
		'er'= read contents in EEPROM for evm build info
		'ew'= write contents of eepromx buffers to EEPROM
		'i' = current delivered by current source
		'x' = IDAC0 output
		'y' = IDAC1 output
		'f' = GPIO input configuration
		'u' = GPIO i/o direction
		'g' = GPIO write outputs
		'j' = GPIO read inputs
		'm' = MUX change for internal measurements
		'h' = Full scale calibration coefficients
		'l' = Offset calibartion coefficients
		'z' = DAC output control
		'id' = ID bits of DUT
		'su' = unipolar/bipolar supply setting
		'rs' = hardware DUT reset
		'pn' = DUT power down
		'vw' = display on LCD
	The various options will need to be defined depending on the options of the ADC. */
	
	
	int i;
	char d_char[7];
	
#ifndef ETEST
	char strbuf[15];
	#else
	char strbuf[36];
	u8 dummy;
	// Resest SPI
	CNTLOUTH();
	
	// Enable communication
	CNTLOUTL();		
	
	while(!(IFG2&UTXIFG1));
		U1TXBUF=0x06;
	while(!(IFG2&URXIFG1));
		dummy=U1RXBUF;
	
	vTaskDelay(2);
	// hardware reset the device
	GPIO1OUTL();
	GPIO1OUTH();
	
	// powerdown and powerup
	GPIO0OUTL();
	vTaskDelay(2);
	GPIO0OUTH();
	
	/* On with the show.....*/
	#ifndef ADS1147
	marquee("   ADS1247EVM TEST V" VERSION " (c)T.I.",0);
	#else
	marquee("   ADS1147EVM TEST V" VERSION " (c)T.I.",0);
	#endif
	
	lcd_allclear();
	
	core_set_task(1);
	core_load_defaults();
	op_mode=JS_MODE_AN;
	config_an();
	vTaskDelay(20);
	
	ads_reg_read();
	
	anyl_stop();
	lcd_allclear();
	aparms.console=1;
	lcd_showstr("PC CONS");
	
	
	#endif
	
	
	
	vTaskDelay(80);
	while (!check_uart());

	uart_init();
	show_version();
	for(;;) {
		
		//uart_txstr("MMB3>");
		#ifndef ADS1147
		uart_txstr("ADS1247EVM>");
		#else
		uart_txstr("ADS1147EVM>");
		#endif
		// Only read the data if there is data in the queue
		while (!uart_have()){
			vTaskDelay(10);
			}
		uart_readln(strbuf,37);
		switch(lcase(strbuf[0])) {
		case 'p':  // gain or power down
			if (strbuf[1]=='n') {
				set_pwrdn(strbuf[2]);
			}
			else set_pga(strbuf[1]);
			break;
		case 'r': // data rate or DUT reset
			if (strbuf[1]=='s') {
				ads_set_reset();
			}
			else set_datarate(lcase(strbuf[1]));
			break;
		case 'b': // buffer enable
			//set_buffer(lcase(strbuf[1]));
			break;
		case 'c': // channel
			if (strbuf[1]=='p') {
			set_ch(strbuf[2],0);
			} else if (strbuf[1]=='m'){
				set_ch(strbuf[2],1);
			} else print("'cp' or 'cm' and 0,1,...3\r\n");
			break;
		case 'v': // version
			if (strbuf[1]=='w') {
				for (i=2;i<9;i++){//if (strbuf[i] == ' ') 
					d_char[i-2]=strbuf[i];
					//else d_char[i-2] = " ";
				}
				lcd_showstr(d_char);
			}
			else {
			show_version();
			core_set_task(1);
			lcd_allclear();
			lcd_showstr("PC CONS");
			}
			break;
		case 's': // stream or supply
			if (strbuf[1]=='u') {
				set_su(strbuf[2]);
			}
			else stream();
			break;
		case 'd': // data
			data(strbuf+1);
			break;
		case 'q': // query
			show_parms();
			break;
		case 'k': // calibrate
			ads_cal();
			vTaskDelay(10);
			if (core_get_task()) {
				lcd_showstr("PC CONS");
			}
			print("ok\r\n");
			break;
		case 'a': // credits
			credits();
			break;
		case 'n': // burn out current source
			set_burnout(strbuf[1]);
			break;
		case 'o': //bias voltage to inputs
			set_bias(strbuf[1],strbuf[2]);
			break;
		case 'w': //reference select
			set_vref(strbuf[1]);
			break;
		case 'e': //internal reference on/off or eeprom read
			switch(lcase(strbuf[1])){
				case 'r':
					eeprom_read();
					break;
				case 'e':
				switch(lcase(strbuf[2])){
				case '1':
					eeprom_write(strbuf,0);
					break;
				case '2':
					eeprom_write(strbuf,0);
					break;
				case '3':
					eeprom_write(strbuf,0);
					break;
				case '4':
					eeprom_write(strbuf,0);
					break;
				case '5':
					eeprom_write(strbuf,0);
					break;
				}
				break;
				
				case 'w':
					eeprom_write(strbuf,1);
					break;
				default:
					set_iref(strbuf[1]);
					break;
			}
			break;
			
		case 'i': //current from current source
			if (strbuf[1]=='d') {
				print((char *)scr_ch_strings[rparms.idac0>>4]);
				uart_txc('\r');
				uart_txc('\n');
			}
			else set_current(strbuf[1]);
			break;
		case 'x': //IDAC0 delivery
			set_idac(strbuf[1],0);
			break;
		case 'y': //IDAC1 delivery
			set_idac(strbuf[1],1);
			break;
		case 'f': //GPIO input configuration
			set_gconfig(strbuf[1],strbuf[2]);
			break;
		case 'u': //GPIO IO direction
			set_gio(strbuf[1],strbuf[2]);
			break;
		case 'g': //GPIO write output
			set_gdat(strbuf[1],strbuf[2]);
			break;
		case 'j': //GPIO read inputs
			get_gdat();
			break;
		case 'm': // MUX position
			set_mux(strbuf[1]);
			break;
		case 't': //test
			//ads1226evm_test();
			break;
		case 'h': //full scale coefficients
			set_full(strbuf[1],strbuf[2],strbuf[3],strbuf[4],strbuf[5],strbuf[6]);
			break;
		case 'l': //offset coefficients
			set_off(strbuf[1],strbuf[2],strbuf[3],strbuf[4],strbuf[5],strbuf[6]);
			break;
		case 'z':  //DAC output
			set_dac(strbuf[1],strbuf[2],strbuf[3],strbuf[4],strbuf[5]);
			break;
		default:
			break;
		}
	}
}

void console_init(int pri)
{
	xTaskCreate(console,"cons",80,0,pri,&this);
}
