
#include "common.h"
#include "task.h"
#include "lcd.h"
#include "joystick.h"
#include "ads.h"
#include "anyl.h"
#include "disp.h"
#include "config.h"

xTaskHandle core_task_handle;
#define this core_task_handle

void core_load_defaults(void);
int core_get_task(void);
void core_set_task(int itask);

u8 op_mode; // KEYS_MODE_AN or KEYS_MODE_SC
/*
static float getone(void)
{
	u8 avgs=anyl_get_avgs();

	anyl_stop();
	anyl_result.newresult=0;
	vTaskDelay(100);
	anyl_set_avgs(8);
	anyl_go();
	while(!anyl_result.newresult)
		vTaskDelay(100);
	anyl_result.newresult=0;
	anyl_set_avgs(avgs);
	return anyl_result.res.f;
}
*/
/* missing scale mode items, such as cal(), checkboth(), tare(), etc called from config_scale()*/
static void range(void)
{
	incunit();
	showunit();
	js_waitrelease();
}

static void display_raw(void)
{
	switch(aparms.unit) {
	case ANYL_UNIT_HEX:
		showhex(anyl_result.res.i);
		break;
	case ANYL_UNIT_DEC:
		showint(anyl_result.res.i);
		break;
	case ANYL_UNIT_VOLT:
		showvolt(code2volt((float)anyl_result.res.i));
		break;

	case ANYL_UNIT_C:
		//showtemp((float)anyl_result.res.i,0);
		break;
	case ANYL_UNIT_F:
		//showtemp((float)anyl_result.res.i,1);
		break;

	default:
		lcd_showstr(" ERROR");
		break;
	}
}

static void display_rms(void)
{
	switch(aparms.unit) {
	case ANYL_UNIT_DEC:
		showdec(anyl_result.res.f);
		break;
	case ANYL_UNIT_VOLT:
		showvolt(code2volt(anyl_result.res.f));
		break;
	case ANYL_UNIT_ENOB:
		showenob(code2enob(anyl_result.res.f));
		break;
	default:
		lcd_showstr(" ERROR");
		break;
	}
}

static void display_avg(void)
{
	switch(aparms.unit) {
	case ANYL_UNIT_DEC:
		showint(anyl_result.res.f);
		break;
	case ANYL_UNIT_VOLT:
		showvolt(code2volt(anyl_result.res.f));
		break;
#ifdef TEMP
	case ANYL_UNIT_C:
		showtemp(anyl_result.res.f,0);
		break;
	case ANYL_UNIT_F:
		showtemp(anyl_result.res.f,1);
		break;
#endif
	default:
		lcd_showstr(" ERROR");
		break;
	}
}

static void display_peak(void)
{
	switch(aparms.unit) {
	case ANYL_UNIT_DEC:
		showdec(anyl_result.res.i);
		break;
	case ANYL_UNIT_VOLT:
		showvolt(code2volt(anyl_result.res.i));
		break;
	case ANYL_UNIT_ENOB:
		showenob(code2enob(anyl_result.res.i));
		break;
	default:
		lcd_showstr(" ERROR");
		break;
	}
}

static void displayanyl(void)
{
	switch(anyl_get_mode()) {
	case ANYL_MODE_RAW:
		display_raw();
		break;
	case ANYL_MODE_RMSN:
		display_rms();
		break;
	case ANYL_MODE_AVG:
		display_avg();
		break;
	case ANYL_MODE_PEAK:
		display_peak();
		break;
	default:
		lcd_showstr("SYSERROR");
		break;
	}
}

static void displaydata(void)
{
	if (anyl_result.newstatus) {
		switch(anyl_result.status) {
		case ANYL_STAT_COLLECTING:
			lcd_showstr("GOT");
			int2asc((u32)anyl_result.collected,3,1,dispbuf);
			lcd_showstr_pos(dispbuf,4);
			return;
		case ANYL_STAT_NODATA:
			lcd_showstr("NODATA");
			return;	
		case ANYL_STAT_DATA:
			break;
		default:
			lcd_showstr("ERRSTAT");
			return;
		}
	anyl_result.newstatus=0;
	if (anyl_result.newresult) {
			displayanyl();
		}
		anyl_result.newresult=0;
	}
}

static void apobar(int ct)
{
	
	lcd_bargraph_set(ct);
	
}


static void showprogress(void)
{
	 if (anyl_get_mode()!=ANYL_MODE_RAW) {
			apobar((int)(((float)anyl_result.collected/anyl_get_avgs())*10));
		} else {
			if (!(state.progdiv++))
				++state.progress;
			if (state.progress>=11)
				state.progress=1;
			
			lcd_bargraph_set(state.progress);
			
		} 
}

/* Need to double check task delay to make sure timing is correct for timeout */
// if timeout != 0, this will return -2 after 0.4 sec with no key press
static int menu_key(int timeout)
{
	js_evt e;

	for(;;) {
		vTaskDelay(10);
		if (timeout&&!js_have())
			return -2;
		while (js_have()) {
			e=js_get();
			if (e.pressed) {
				if (js_pressed(JS_SELECT)) {
						return -1;
				}
			}
			if (!e.pressed&&!e.held) {
				switch(e.jssw) {
				case JS_UP:
					return JS_UP;
				case JS_DOWN:
					return JS_DOWN;
				case JS_RIGHT:
					return JS_RIGHT;
				case JS_LEFT:
					return JS_LEFT;
				/*case JS_PARMUP:
					screen_next();
					return -1;
				case JS_PARMDN:
					screen_prev();
					return -1;
					*/
				default:
					break;
				}
			}
		}
	}
}
/* returns -2 if no key is pressed after 0.4sec if timeout is non-zero.
returns -1 if some kind of mode change happened
returns 0 if ENT or SEL is pressed
if timeout is 0 this blocks until a key is pressed
*/

static void switchmode(void)
{
	u8 mode=js_modesw();
	if (mode==op_mode) return;
	if (mode==JS_MODE_AN)
		config_an();
	else
		config_sc();
	op_mode=mode;
}

static void quick_show_unit(void)
{
	chunit();
	showmode();
	js_waitrelease();
	switchmode();
	anyl_result.newresult=1;
}



static void menu_items(void)
{
	/* Main menu items are listed and when function is called up/down on
	joystick moves through the list.  Selecting on the joystick takes you 
	back out of the menu mode.  Pressing right/left steps you through the 
	submenus */
	const int menu_count=4;
	const char *menu_val[]={"DISPLAY","CONFIG ","PARAMS ","DAC    "};
	int sel;
	
	anyl_stop();
	sel=0;
	
	for (;;) {
		
		
		lcd_showstr((char *)menu_val[sel]);
		
		lcd_arrowset(1,0);	//up
		lcd_arrowset(3,1);	//down
		lcd_arrowset(2,1);	//right	
		switch(menu_key(0)) {
		case JS_DOWN:
			++sel;
			if (sel>=menu_count) {
				sel=0;
			}
			break;
		case JS_UP:
			--sel;
			if (sel<0) sel=menu_count-1;
			break;
		case JS_RIGHT:
			switch (sel) {
			case 0:			// Display Menu
				dispsetup(); 	// calls found in qconfig.c
				anyl_result.newresult=1;
				switchmode();
				break;
			case 1:			// Config Menu
				config(); 	// calls found in config.c
				anyl_result.newresult=1;
				switchmode();
				break;
			case 2:			// Params Menu
				quickconfig();
				anyl_result.newresult=1;
				switchmode();
				break;
			case 3:			// DAC Menu
				break;
			default:
				break;
			}
			break;
		case -1:
			return;
		default:
			break;
		}
	}
	
}


static void core_anyl(u8 jssw)
{
	switch(jssw) {
	case JS_UP:
		lcd_showstr("UP");
		
		break;
	case JS_DOWN:
		lcd_showstr("DOWN");
		
		break;
	case JS_SELECT:
		lcd_showstr("SELECT");
		menu_items();
		lcd_allclear();
		//config();
		config_an();
		anyl_go();
		break;
	case JS_LEFT:
		//ads_write();
	//quick_show_unit();
		//lcd_showstr("LEFT");
		break;
	case JS_RIGHT:
		//ads_read();
	//anyl_restart();
		//lcd_showstr("RIGHT");
		break;
	default:
		break;
	}
}
int core_get_task(void)
{
	return aparms.console;
}

void core_set_task(int itask)
{	aparms.console = itask;
}

static void core_scale(u8 jssw)
{
	switch(jssw) {
	case JS_UP:
		lcd_showstr("UP");
		
		break;
	case JS_DOWN:
		lcd_showstr("DOWN");
		
		break;
	case JS_SELECT:
		lcd_showstr("SELECT");
		//menu_items();
		break;
	case JS_LEFT:
		lcd_showstr("LEFT");
		break;
	case JS_RIGHT:
		lcd_showstr("RIGHT");
		break;
	default:
		break;
	}
}

void core_load_defaults(void)
{
	disp_vref=2.048;
	//disp_vref=5.0;
	aparms.console=0;
	/*//sparms.unit=UNIT_SI;
	sparms.calunit=SCALE_K;
	sparms.unit=SCALE_K;

	sparms.ch=0;

	sparms.tare=0;
	sparms.disp_avgs=4;
	sparms.m=0;
	sparms.w_fs=5000;
	sparms.w_zs=0;
	sparms.speed=0;
*/
	aparms.speed=0;

	aparms.ch_p=0;
	aparms.ch_m=1;
	aparms.tempunit=0;
	aparms.burnout=0;
	aparms.refsel=0;
	aparms.refin=0;
	aparms.current=0;
	aparms.idac0=10;
	aparms.idac1=10;
	aparms.gconfig=0x00;
	aparms.gdir=0x0f;
	aparms.gpio=0x00;
	aparms.mux=0;
	aparms.pga=0;
	aparms.unit=0;
	aparms.supply=1;
	aparms.pwrdn=1;
	anyl_set_mode(0);
	aparms.avgs=50;
	// default register settings
	rparms.mux0=0x01;
	rparms.vbias=0x00;
	rparms.mux1=0x00;
	rparms.sys0=0x00;
	rparms.ofc0=0x00;
	rparms.ofc1=0x00;
	rparms.ofc2=0x00;
	rparms.fsc0=0x00;
	rparms.fsc1=0x00;
	rparms.fsc2=0x40;
	rparms.idac0=0x00;
	rparms.idac1=0xff;
	rparms.gpiocfg=0x00;
	rparms.gpiodir=0x0f;
	rparms.gpiodat=0x00;
	
}


static void core(void *p)
{
	/* There are a number of options that could be set or displayed here 
	in this functiion, but not sure if this is the best place.....for example....
	marquee of the firmware loaded.....or if calibration or initializations calls should be 
	made here.  Perhaps the calls should be made, but the function resides in 'ads.c' to include calls
	like 'ads_cal(), etc.. */
	char buf[9];
	js_evt e;
	u8 dummy;
	
	/* As per ADS1248 datasheet, there should be a reset command given at power up.  
	There needs to be a minimum of 2 SPI clocks and hardware reset.  Doing it here instead
	of ads_init() to make use of vTaskDelay() which is a part of RTOS and it hasn't been
	started yet when ads_init() is called from main.*/
	// Resest SPI
	CNTLOUTH();
	
	// Enable communication
	CNTLOUTL();		
	
	while(!(IFG2&UTXIFG1));
		U1TXBUF=0x06;
	while(!(IFG2&URXIFG1));
		dummy=U1RXBUF;
	
	vTaskDelay(2);
	// hardware reset the device
	GPIO1OUTL();
	GPIO1OUTH();
	
	// powerdown and powerup
	GPIO0OUTL();
	vTaskDelay(2);
	GPIO0OUTH();
	
	/* On with the show.....*/
	//marquee("   ADS####EVM V" VERSION " (c)T.I.",0);
	#ifndef ADS1147
	//marquee("   ADS####EVM V" VERSION " (c)T.I.",0);
	
	marquee("   ADS1247EVM V" VERSION " (c)T.I.",0);
	
	#else
	
	marquee("   ADS1147EVM V" VERSION " (c)T.I.",0);
	
	#endif
	
	
	lcd_allclear();
	js_set_monitor(this);
	anyl_set_monitor(this);
	vTaskDelay(200);
	//ads_cal();
	op_mode=100;
	anyl_result.status=ANYL_STAT_NODATA; // 'cause anyl won't do it
	//if (flash_load_parms()) core_load_defaults();
	core_load_defaults();
	switchmode();
	
	ads_reg_read();
	if (!aparms.console) {
	anyl_go();
	
	ads_cal();
	}
	else {
		anyl_stop();
		lcd_showstr("PC CONS");
	}
	
	for(;;) {
		while(js_have()) {	//anything in the queue?
			e=js_get();	//then go get it
			if (!aparms.console) {
			if (e.pressed){
				if (op_mode==JS_MODE_AN) 
					core_anyl(e.jssw);
				else 
					core_scale(e.jssw);
				}
			}
			}
			if (!aparms.console) {
				displaydata();
				showprogress();
				vTaskDelay(1) ;
			}
			else vTaskDelay(1000);
			
		}
}

void core_init(void)
{
	/* This section of code actually sets various parameters.  This is old code 
	and should be a part of the flash check routine to see if the values have been
	saved to flash.  Currently this will need to be updated here...but should have a call
	to another function for setup.  This happens in core() */
	
	state.qscreen=0;
	state.ascreen=0;
	state.sscreen=0;
	
	
	xTaskCreate(core,"core",110,0,0,&this);
}


