
#include "common.h"
#include "task.h"
#include "list.h"
#include "joystick.h"
#include "lcd.h"

#define tskSTACK_FILL_BYTE	( 0xa5 )

size_t mem_bytes_used(void);

typedef struct tskTaskControlBlock
{
	portSTACK_TYPE			*pxTopOfStack;		/*< Points to the location of the last item placed on the tasks stack.  THIS MUST BE THE FIRST MEMBER OF THE STRUCT. */
	portSTACK_TYPE			*pxStack;			/*< Points to the start of the stack. */
	unsigned portBASE_TYPE	uxTCBNumber;		/*< This is used for tracing the scheduler and making debugging easier only. */
	unsigned portBASE_TYPE	uxPriority;			/*< The priority of the task where 0 is the lowest priority. */
	xListItem				xGenericListItem;	/*< List item used to place the TCB in ready and blocked queues. */
	xListItem				xEventListItem;		/*< List item used to place the TCB in event lists. */
	signed portCHAR			pcTaskName[ configMAX_TASK_NAME_LEN ];/*< Descriptive name given to the task when created.  Facilitates debugging only. */
	unsigned portSHORT		usStackDepth;		/*< Total depth of the stack (when empty).  This is defined as the number of variables the stack can hold, not the number of bytes. */
} tskTCB;

static u16 free_stack(xTaskHandle t)
{
	register u16 usCount = 0;
	u8 *pucStackByte;

	taskENTER_CRITICAL();
	pucStackByte=(u8 *)(((tskTCB *)t)->pxStack);

	while( *pucStackByte == tskSTACK_FILL_BYTE )
	{
		pucStackByte -= portSTACK_GROWTH;
		usCount++;
	}
	taskEXIT_CRITICAL();

	usCount /= sizeof( portSTACK_TYPE );

	return usCount;
}

#define SCREEN_COUNT 4

static int screen;
static char cbuf[5];

void scr_debug(void)
{
	u16 n;
	js_evt e;

	screen=0;
	for(;;) {
		if (js_have()) {
			e=js_get();
			if (e.pressed&&!e.held) {
				switch(e.jssw) {
				case JS_PARMUP:
					--screen;
					if (screen<0) screen=SCREEN_COUNT-1;
					break;
				case JS_PARMDN:
					++screen;
					if (screen>=SCREEN_COUNT) screen=0;
					break;
				case JS_SEL:
					return;
				default:
					break;
				}
			}
		}
		switch(screen) {
		case 0: // memory
			lcd_showstr("MEM");
			//int2asc(mem_bytes_used(),4,1,cbuf);
			//int2asc(42,4,0,cbuf);
			n=mem_bytes_used();
			break;
		case 1: // core stack
			lcd_showstr("CORE");
			n=free_stack(core_task_handle);
			break;
		case 2: // keys stack
			lcd_showstr("KEYS");
			n=free_stack(js_task_handle);
			break;
		case 3: // anyl stack
			lcd_showstr("ANYL");
			n=free_stack(anyl_task_handle);
			break;
		default:
			screen=0;
			break;
		}
		hex2asc(n,4,1,cbuf);
		lcd_showstr_pos(cbuf,4);
		vTaskDelay(600);
	}
}
