
#include "disp.h"
#include "lcd.h"
#include "nconv.h"
#include "ads.h"
#include "config.h"
#include "joystick.h"
#include <string.h>
#include <math.h>

float disp_vref;

float code2volt(float c)
{
	/* For the ADS1230/32REF, the formula from the datasheet uses
	.5VREF/((1L<<23)-1) this equates to VREF/((1L<<24)-1)
	
	For the ADS1248EVM, the formula from the datasheet uses
	VREF/(1L<<23)-1)/PGA   */
	#ifndef ADS1147
	if (c>0x7FFFFF){
		c=-c;
	};
	return ((c/((1L<<23)-1)))*(disp_vref/ads_get_gain());
	#else
	if (c>0x7FFF){
		c=-c;
	};
	return ((c/((1L<<15)-1)))*(disp_vref/ads_get_gain());
	#endif
	
}

float code2enob(float c)
{
	if (c==0) return 24.0;
	return 24.0-log2f(c);
}

u32 abs32(s32 i)
{
	if (i<0) return (u32)(-i);
	else return (u32)(i);
}

void showvolt(float volts)
{
	float absv;
	char unit;
	int digs;

	absv=fabs(volts);
	if (absv>=1.0) {
		unit='V';
		digs=1;
	} else if (absv>=0.001) {
		unit='m';
		volts*=1000;
		digs=3;
	} else if (absv>=1e-6) {
		unit='u';
		volts*=1e6;
		digs=3;
	} else {
		unit='n';
		volts*=1e9;
		digs=3;
	}
	f2asc(volts,digs,6-digs,0x81,dispbuf);
	dispbuf[8]=unit;
	dispbuf[9]=0;
	lcd_showstr(dispbuf);
	
}

void showenob(float enob)
{
	if (enob==24.0) {
		lcd_showstr("24 (MAX)");
	} else {
		f2asc(enob,2,1,0x81,dispbuf);
		dispbuf[5]=' ';
		dispbuf[6]='B';
		dispbuf[7]='I';
		dispbuf[8]='T';
		dispbuf[9]=0;
		lcd_showstr(dispbuf);
	}
}

void showhex(s32 code)
{
	s32 c=code;

	//dispbuf[0]=' ';
	#ifndef ADS1147
	hex2asc(&c,6,2,dispbuf);
	#else
	hex2asc(&c,4,2,dispbuf);
	dispbuf[4]=' ';
	dispbuf[5]=' ';
	#endif
	dispbuf[6]='h';
	dispbuf[8]=0;
	lcd_showstr(dispbuf);
}

void showint(s32 code)
{
	
	float absc;
	int i=0;
	absc=abs32(code);
	
	int2asc(absc,7,1,dispbuf);
	while (i<9){
		dispbuf[9-i]=dispbuf[8-i];
		i++;
	}
	dispbuf[0]=(code<0)?'-':'+';
	lcd_showstr(dispbuf);
}

void showdec(float code)
{
	char unit;
	int digs;
	float abscode=fabsf(code);

	if (abscode>=1e6)
		digs=7;
		
	else if (abscode>=1e3)
		digs=6;
	else
		digs=3;
		
	f2asc(code,digs,6-digs,0x81,dispbuf);
	lcd_showstr(dispbuf);
}

static int checkjs(void)
{
	js_evt e;

	if (js_have()) {
		e=js_get();
	}
	return 0;
}

void marquee(char *s, int jssw)
{
	int i;
	char *str;

	lcd_allclear();
	if (jssw)
		js_waitrelease();
	for (;;) {
		str=s;
		for (i=6;i>0;--i) {
			lcd_showstr_pos(str,i);
			vTaskDelay(MARQUEE_SPEED);
			if (jssw&&checkjs()) return;
		}
		while (*str) {
			lcd_showstr(str);
			vTaskDelay(MARQUEE_SPEED);
			if (jssw&&checkjs()) return;
			++str;
			if (*str=='.') ++str;
		}
		lcd_allclear();
		if (!jssw) return;
	}
}
/* Not clear as to why this was seperated out with a TEMP compiler directive.  Left
in as the function could be useful for displaying temps from internal temp sensor or
temp sensor parts (EVMs) */

/*
#ifdef TEMP

#define MT 0.000379
#define CT 0.1022

void showtemp(float c, int f)
{
	float t;

	t=(code2volt(c)-CT)/MT;
	if (f) t=(9.0/5.0)*t+32.0;
	f2asc(t,3,3,1,dispbuf);
	dispbuf[7]='\177';
	dispbuf[8]=f?'F':'C';
	dispbuf[9]=0;
	lcd_showstr(dispbuf);
}

#endif

*/
