
#ifndef GUARD_flash_h
#define GUARD_flash_h

void flash_save_parms();

/* Loads all operating parameters from flash memory.

The data is stored in information flash and arranged as follows:

0x1100 -- checksum, 16 bit XOR
0x1102 -- flag word: must be 0xabcd (little endian)
0x1104 -- vref (float32, little-endian)
0x1108 -- aparms struct
   ?   -- sparms struct (@ 0x1108 + sizeof(anyl_parms_t))

The flag word is checked first and included in the checksum.  It must be 
0xABCD.  This is included because the memory erases to 0xFFFF, and an odd 
number of words sums to 0xFFFF, so the checksum would pass on erased flash.

The checksum is all of the words XORed together.  When XORed with the 
checksum word, the result must be zero.

If the flag word or the checksum do not match, defaults are loaded.
*/
int flash_load_parms();

void flash_erase_info();

/* writes \p len words of \p data to flash memory at \p adr. */
void flash_write(short adr, short* data, short len);

void flash_read(short adr, short* data, short len);

void flash_init();

#endif
