
#include <msp430x44x.h>
#include "i2c.h"

#define I2C_SDAA_PORT port1
#define I2C_SDAA_PIN pin1
#define I2C_SCLA_PORT port1
#define I2C_SCLA_PIN pin0

#define I2C_SDAA_DIR I2C_SDAA_PORT.dir.I2C_SDAA_PIN
#define I2C_SCLA_DIR I2C_SCLA_PORT.dir.I2C_SCLA_PIN
#define I2C_SDAA_IN I2C_SCLA_PORT.in.I2C_SDAA_PIN
#define I2C_SCLA_IN I2C_SCLA_PORT.in.I2C_SCLA_PIN

#define I2C_SDAA(MODE) I2C_SDAA_PORT.MODE.I2C_SDAA_PIN
#define I2C_SCLA(MODE) I2C_SCLA_PORT.MODE.I2C_SCLA_PIN

#define DELAY(c) busywait(c)
#define SDL 4

#define I2C_SDAAH() I2C_SDAA_DIR=0; DELAY(SDL)
#define I2C_SDAAL() I2C_SDAA_DIR=1; DELAY(SDL)
#define I2C_SCLAH() I2C_SCLA_DIR=0; DELAY(SDL)
#define I2C_SCLAL() I2C_SCLA_DIR=1; DELAY(SDL)
#define I2C_SDAAR() I2C_SDAA_IN; DELAY(SDL)
#define I2C_SCLAR() I2C_SCLA_IN; DELAY(SDL)

static void busywait(int c)
{
	int i;
	
	for(i=c;i;i--) __asm__ __volatile__("; loop %0": "+r" (i));
}


void i2cStart(void)
{
	I2C_SDAAH();
	I2C_SCLAH();
	I2C_SDAAL();
}

void i2cStop(void)
{
	I2C_SDAAL();
	I2C_SCLAH();
	I2C_SDAAH();
}

int i2cWByte(u8 b)
{
	int i;

	for (i=0;i<8;++i) {
		I2C_SCLAL();
		if (b&0x80) {
			I2C_SDAAH();
		}
		else {
			I2C_SDAAL();
		}
		I2C_SCLAH();
		b=b<<1;
	}
	I2C_SCLAL();
	I2C_SDAAH();
	I2C_SCLAH();
	i=I2C_SDAAR();
	I2C_SCLAL();
	return !i;
}

void i2cRByte(u8 *b, int ack)
{
	int i,i2;
	
	*b=0;
	I2C_SCLAL();
	I2C_SDAAH();
	for (i=0;i<8;++i) {
		I2C_SCLAL();
		DELAY(2);
		I2C_SCLAH();
		DELAY(2);
		i2=I2C_SDAAR();
		*b=*b<<1;
		*b|=i2?1:0;
	}
	I2C_SCLAL();
	DELAY(2);
	if (ack) {
		I2C_SDAAL();
	}
	DELAY(2);
	I2C_SCLAH();
	DELAY(2);
	I2C_SCLAL();
}

I2CErr i2cWrite(u8 adr, u8 *buf, int len)
{
	int i;

	i2cStart();
	if (!i2cWByte(adr&0xFE))
		return I2C_ERR_NAKADR;
	for (i=0;i<len;++i) {
		if (!i2cWByte(*buf))
			return I2C_ERR_NAKWR;
		++buf;
	}
	i2cStop();
	return I2C_ERR_OK;
}

I2CErr i2cRead(u8 adr, u8 *buf, int len)
{
	int i;

	i2cStart();
	if (!i2cWByte(adr|1))
		return I2C_ERR_NAKADR;
	for (i=1;i<=len;++i)
		i2cRByte(buf++,i!=len);
	i2cStop();
	return I2C_ERR_OK;
}

I2CErr i2cGCReset(void)
{
	i2cStop();
	i2cStart();
	i2cWByte(0);
	i2cWByte(6);
	i2cStop();
	return I2C_ERR_OK;
}

void i2cInit(void)
{
	I2C_SDAA(ie)=0;
	I2C_SCLA(ie)=0;
	I2C_SDAA(sel)=0;
	I2C_SCLA(sel)=0;
	I2C_SDAAH();
	I2C_SDAA(out)=0;
	I2C_SCLAH();
	I2C_SCLA(out)=0;

//	i2cGCReset();
}
