
#ifndef GUARD_joystick_h
#define GUARD_joystick_h

#include "common.h"
#include "task.h"

/* Joystick task

Scans switches roughly every 25msec.

Presses are reported through a queue.  Queue elements are small structs which contain a switch code, a pressed flag, and a held flag.  
The switch code says which switch it is.  The status says whether the switch is pressed or released.  The held flag says whether the 
switch has been pressed for longer than 1 sec.

When a switch is pressed, you get a pressed event with a zero time.  If the switch is held, you get one more event saying that it was 
held down.  When a switch is released, you get a released event.

*/

/*
	Joystick mapping:
	
	P1.3 - Down
	P1.4 - Select
	P1.5 - Up
	P1.6 - Left
	P1.7 - Right

*/


#define JS_NONE 0
#define JS_DOWN 1
#define JS_SELECT 2
#define JS_UP 3
#define JS_LEFT 4
#define JS_RIGHT 5

//#define KEYS_RANGE 1
//#define KEYS_TARE 2
//#define KEYS_ENT 2
#define JS_SEL 2
#define JS_PARMUP 3
#define JS_PARMDN 1
#define JS_VALUP 3
#define JS_VALDN 1
//#define KEYS_CAL 3
#define JS_MODE_SC 6
#define JS_MODE_AN 7

typedef struct js_evt {
	unsigned int pressed:1,
		jssw:3,
		held:1;
} js_evt;

/*!
\param pri Task priority
*/
void js_init(u8 pri);

/*!
\param monitor Task to wake when a joystick event occurs.  vTaskResume() is called to do the wakeup.
*/
void js_set_monitor(xTaskHandle monitor);

/*! If a switch has been pressed, returns its event.  If no switch has been 
pressed, returns JS_NONE.  Does not block.
*/
js_evt js_get(void);

//! True if a switch has been pressed, false if not
int js_have(void);

//! Waits indefinitely for a switch
js_evt js_block(void);

//! Return 1 if the given switch is pressed
int js_pressed(int jssw);

//! Returns 1 if any switch is currently pressed, 0 otherwise
int js_anypressed(void);

//! Waits for all switches to be released; blocks indefinitely
void js_waitrelease(void);

//! Clears the joystick event queue, discarding any events in it
void js_flush(void);

//! Check mode switch position
/*! Returns either JS_MODE_SC (scale mode) or JS_MODE_AN (analysis mode) */
int js_modesw(void);

#endif
