
#ifndef GUARD_lcd_h
#define GUARD_lcd_h

#include "common.h"

#define lcd_dp_on(ind) lcd_dp_set(ind,1)
#define lcd_dp_off(ind) lcd_dp_set(ind,0)

extern char dispbuf[10];

/*! Arrow definition: Left = 0, Up = 1, Right = 2, Down = 3.  If show, leaves current
selection of arrows on otherwise turns them all off. */
void lcd_arrowset(u8 arrow, u8 show);

/*! Displays the antenna symbol.  On if show otherwise off. */
void lcd_antenna(u8 show);

/*! Displays signal strength bar graph.  Strength goes from 0 to 3; 0 turns 
indicator off, 1-3 light up segments of bar graph. */
void lcd_ss(u8 strength);

/*! Displays the battery symbol.  On if show, otherwise off. */
void lcd_battery_show(u8 show);

/*! Displays battery strength segments.  Value goes from 0 to 3; 0 turns bars off,
1-3 light up segments. */
void lcd_battery_set(u8 value);

/*! Displays the bargraph symbol.  On if show, otherwise off. */
void lcd_bargraph_show(u8 show);

/*! Displays bargraph segments. Value goes from 0 to 10; 0 turns bars off, 
1-10 light up segments. */
void lcd_bargraph_set(u8 value);

/*! Clears everything on the LCD.....Everything! */
void lcd_allclear(void);

/*! Clears all 7 starburst characters on LCD. */
void lcd_clear();

/*! Turns on all possible display segements.....Everything on the LCD. */
void lcd_allon(void);

/*! Displays a single character at given position. */
void lcd_showc(char c, int position);
/*! Just a shortcut to display a string after clearing the LCD starting at the 
first LCD character position. */
#define lcd_showstr(C) lcd_clear(); lcd_showstr_pos((C),1)

/*! Display characters pointed by c starting at LCD position pos. */
void lcd_showstr_pos(char *c, int pos);

/*! Displays decimal points at position ind.  If on, turns it on otherwise off. */
void lcd_dp_set(int ind, int on);


void lcd_test(void);

/*! Sets timer and segments to initial conditions. */
void lcd_init(void);

#endif
