
#include "nconv.h"

#define h2a(d) ((d>9)?(d+'A'-10):(d+'0'))

#define LITTLEENDIAN 1
int hex2asc(void *npos, int digs, int mode, char *s)
{
	int i,zero;
	char dig;
	char *spos=s;
	char *n=(char *)npos;

	zero=1;
#if LITTLEENDIAN
	n+=(digs-1)>>1;
#else
	n+=(16-digs)>>1;
#endif
	for (i=digs-1;i>=0;--i) {
		if (i&1) {
			dig=(*(char *)n>>4)&15;
		} else {
			dig=*(char *)n&15;
#if LITTLEENDIAN
			--n;
#else
			++n;
#endif
		}
		if (zero&&dig)
			zero=0;
		if (zero) {
			switch(mode) {
			case 1:
				*spos++=' ';
				break;
			case 2:
				*spos++='0';
				break;
			default:
				break;
			}
		} else
			*spos++=h2a(dig);
	}
	if (zero&&mode==1)
		*(spos-1)='0';
	else if (zero&&mode==0)
		*spos++='0';
	*spos=0;
	return spos-s;
}

int int2asc(u32 n, int digs, int mode, char *s)
{
	u64 acc=0;
	u64 bw=1;
	int i;

	for (i=0;i<32;++i) {
		if (n&1)
			bcd64add(&bw, &acc, 0);
		bcd64add(&bw,&bw,0);
		n=n>>1;
		if (!n) break;
	}
	return hex2asc(&acc,digs,mode,s);
}

void frac2asc(u32 n, int digs, char *s)
{
	u64 bw=0x5000000000000000;
	u64 acc=0;
	int i;
	char *digp;
	char dig;

	for (i=0;i<32;++i) {
		if (n&0x80000000)
			bcd64add(&bw,&acc,0);
		bcd64div2(&bw);
		n=n<<1;
		if (!n) break;
	}
#if LITTLEENDIAN
	digp=(char *)(&acc)+7;
#else
	digp=(char *)(&acc);
#endif
	for (i=0;i<digs;++i) {
		if (!(i&1))
			dig=(*digp>>4)&15;
		else {
			dig=*digp&15;
#if LITTLEENDIAN
			--digp;
#else
			++digp;
#endif
		}
		*s++=h2a(dig);
	}
	*s=0;
}

int fixed2asc(u32 n, int bp, int idigs, int fdigs, int mode, char *s)
{
	char *spos=s;

	if (idigs) {
		if (bp<32)
			spos+=int2asc(n>>bp,idigs,mode,s);
		else
			spos+=int2asc(0,idigs,mode,s);
	}
	if (fdigs>0) {
		*spos++='.';
		frac2asc(n<<(32-bp),fdigs,spos);
		spos+=fdigs;
	}
	return spos-s;
}

int f2asc(float n, int idigs, int fdigs, char mode, char *s)
{
	unsigned long mantissa;
	unsigned char exp;
	char *spos=s;
	int i;
	int bp;

	mantissa=*((unsigned long *)&n);
	// print sign
	if (mantissa&0x80000000)
		*spos++='-';
	else if (mode&0x80)
		*spos++='+';

	// special case: zero
	if (!(mantissa&0x7fffffff)) {
		switch (mode&3) {
		case 1:
			exp=' ';
			break;
		case 2:
			exp='0';
			break;
		default:
			exp=0;
			break;
		}
		if (exp)
			for (i=0;i<idigs;++i)
				*spos++=exp;
		if (idigs) {
			if ((mode&3)==1)
				*(spos-1)='0';
			else if ((mode&3)==0)
				*spos++='0';
		}
		if (fdigs) {
			*spos++='.';
			for (i=0;i<fdigs;++i)
				*spos++='0';
		}
		*spos=0;
		return spos-s;
	}

	// extract mantissa & exp
	exp=(mantissa>>23)&0xff;
	mantissa&=0x007fffff;

	// check that exp is in range
	if (exp<95) {
		*spos++='S';
		*spos++='M';
		*spos=0;
		return spos-s;
	}
	if (exp>158) {
		*spos++='L';
		*spos++='G';
		*spos=0;
		return spos-s;
	}

	// normalise mantissa
	mantissa|=0x00800000;
	bp=32-(exp-118); // max convertible bp is 32+23 = 55
	if (bp<0) {
		mantissa=mantissa<<(-bp);
		bp=0;
	} else if (bp>32) {
		mantissa=mantissa>>(bp-32);
		bp=32;
	}
	
	spos+=fixed2asc(mantissa,bp,idigs,fdigs,mode&3,spos);
	return spos-s;
}

void bcd2tochars(char n, char *s)
{
	s[1]=(n&15)+'0';
	s[0]=((n>>4)&15)+'0';
}

void bcd16tostr(u64 *n, char *c)
{
	char *np=(char *)n+7;
	int i;

	for (i=0;i<8;++i) {
		bcd2tochars(*np,c);
		c+=2;
		np-=1;
	}
}
