
#include "uart.h"
#include "common.h"
#include "queue.h"
#include "config.h"
#include <signal.h>

static xQueueHandle iq;

void uart_init(void)
{
	// set up queues
	iq=xQueueCreate(37,1);

//	USART0 setting for UART mode
	P2SEL |= 0x30;					// P2.4,5 = USART0 TXD/RXD
	ME1 |= UTXE0 + URXE0;			// Enable USART0 TXD/RXD
	UCTL0 |= CHAR;					// 8-bit character
	UTCTL0 |= SSEL1;				// UCLK = SMCLK

	UBR10 = 0x00;					// 1MHz 9600 / 1MHz 115200 / 4MHz 115200
	/* 4MHz settings*/
	/*
	UBR00 = 0x23;					// 0x23:4.096MHz 115200
	UMCTL0 = 0xD6;					// 0xD6:modulation for 4.096MHz 115200
	*/
	/* 8MHz settings */
	//UBR00 = 0x45;					// 0x45:8MHz 115200
	//UMCTL0 = 0x29;					// 0x29:modulation for 8MHz 115200
	
	UBR00 = 0x22;					// 0x22:8MHz 230400
	UMCTL0 = 0x5B;					// 0x03:modulation for 8MHz 230400
	
	UCTL0 &= ~SWRST;				// Initialize USART state machine
	IE1 |= URXIE0;					// Enable USART0 RX interrupt
	P2DIR |= 0x10;					// P2.4 output direction
}

interrupt (USART0RX_VECTOR) uart0rx_isr(void)
{
	char c;
	
	c=U0RXBUF;
	
	xQueueSendFromISR(iq,&c,pdFALSE);
		//vtaskYIELD();
	
}

void uart_txc(char c)
{
	while (!(IFG1&UTXIFG0));
	U0TXBUF=c;
}

void uart_txstr(char *c)
{
	while (*c) uart_txc(*(c++));
}

int uart_have(void)
{
	return uxQueueMessagesWaiting(iq)>0;
}

int uart_get_timeout(char *c, int timeout)
{
	return xQueueReceive(iq,c,timeout);
}

char uart_get(void)
{
	char c;
	while (!uart_get_timeout(&c,1000));
	return c;
}

void uart_readln(char *s, int len)
{
	char *pos;
	int nlrx=0;

	pos=s;
	DRDYIEL();
	while (nlrx<1) {
		// go get a character
		*pos=uart_get();
		// if the character is a CR identify it as a LF
		if (*pos=='\r') *pos='\n';
		else if ((pos-s)>=len) continue;
		switch (*pos) {
			// if we see a LF echo CRLF
		case '\n':
			++nlrx;
			uart_txc('\r');
			uart_txc('\n');
			++pos;
			break;
		case '\b':
		case 0x7f:
			if (pos>s) {
				--pos;
				uart_txc('\b');
				uart_txc(' ');
				uart_txc('\b');
			}
			break;
			// we echo the received character by default
		default:
			uart_txc(*pos);
			++pos;
			break;
		}
	}
	if (!aparms.console){
		// clear and reenable int
	DRDYIFGL();
	DRDYIEH();
	 
	}
	
	
	if (nlrx)
		*(pos-1)=0;
	*pos=0;
}
